package com.weaver.seconddev.cockpit.cmd;

import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.ebuilder.datasource.api.enums.SqlParamType;
import com.weaver.seconddev.cockpit.util.DatabaseUtils;
import com.weaver.teams.domain.user.SimpleEmployee;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 流程、日程、协助、文档 4个类型数据
 *
 * @author shil
 * @version 1.00版本
 * @Date 2024/5/22
 */
@Component
public class CockpitBlogDataCmd {
    //日志
    private final static Logger log = LoggerFactory.getLogger(CockpitBlogDataCmd.class);

    @Autowired
    private DatabaseUtils databaseUtils;


    public Map<String, Object> getDataInfo(SimpleEmployee simpleEmployee, Map<String, Object> paramMap,String tenantKey) {

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Map<String, Object> recordMap = new HashMap<String, Object>();

        try {
            String employeeId = String.valueOf(simpleEmployee.getEmployeeId());

            String groupId = "weaver-elog-service";
            String sourceType = "LOGIC";

            String sql =" select w.id,w.blog_time,w.content \n" +
                        " from (\n" +
                        "  select id,blog_time,content \n" +
                        "  from blog where creator = ?\n" +
                        "  and delete_type = 0\n" +
                        "  and tenant_key = ? \n" +
                        "  order by blog_time desc\n" +
                        " ) w limit 5 " ;

            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.LONG);
            sqlParamEntity.setValue(employeeId+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);

            List<Map<String,Object>> blogList = new ArrayList<Map<String,Object>>();
            Map<String,Object> blogMap = null;

            if(recordList.size()>0){
                for(int i=0;i<recordList.size();i++){
                    Map<String,Object> dataMap = recordList.get(i) ;
                    String blogid = String.valueOf(dataMap.get("id"));
                    String blog_time = String.valueOf(dataMap.get("blog_time"));
                    String content = String.valueOf(dataMap.get("content"));
                    String blogDate = "";
                    if(StringUtils.isNotBlank(blog_time)){
                        try{
                            Long blogTime = Long.parseLong(blog_time);
                            Date date = new Date(blogTime);
                            blogDate = sdf.format(date);
                        }catch (Exception e){

                        }
                    }

                    if(StringUtils.isNotBlank(blogDate)){
                        blogMap = new HashMap<String,Object>();
                        blogMap.put("blogid",blogid);
                        blogMap.put("blog_time",blogDate);
                        blogMap.put("content",content);
                        blogList.add(blogMap);
                    }
                }
            }
            recordMap.put("datas",blogList);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("PortalPerformanceInfoCmd :" + e);
        }
        return recordMap;
    }
}
