package com.weaver.seconddev.cockpit.cmd;

import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.ebuilder.datasource.api.enums.SqlParamType;
import com.weaver.framework.rpc.annotation.RpcReference;
import com.weaver.seconddev.cockpit.util.DatabaseUtils;
import com.weaver.seconddev.cockpit.util.JucailinCockpitUtils;
import com.weaver.teams.client.doc.remote.DocClientService;
import com.weaver.teams.domain.user.SimpleEmployee;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.util.*;

/**
 * 文档分析
 *
 * @author shil
 * @version 1.00版本
 * @Date 2024/5/22
 */
@Component
public class CockpitDocumentAnalysisCmd {
    //日志
    private final static Logger log = LoggerFactory.getLogger(CockpitDocumentAnalysisCmd.class);

    @Autowired
    private JucailinCockpitUtils jucailinCockpitUtils;

    @Autowired
    private DatabaseUtils databaseUtils;

    @RpcReference
    DocClientService docClientService;


    public Map<String, Object> getDataInfo(SimpleEmployee employee, Map<String, Object> paramMap,String tenantKey) {

        Map<String, Object> recordMap = new HashMap<String, Object>();
        try {
            List<Map<String,String>> dataList = getDocumentDate(employee,tenantKey);
            recordMap.put("leftdata",dataList);
            List<Map<String, Object>> rightList = getRightViewData(employee,tenantKey);
            recordMap.put("rightdata",rightList);;
        } catch (Exception e) {
            e.printStackTrace();
            log.error("CockpitDocumentAnalysisCmd-e:" + e);
        }
        return recordMap;
    }

    /***
     *
     * @param employee
     * @param tenantKey
     * @return
     */
    public List<Map<String,String>> getDocumentDate(SimpleEmployee employee,String tenantKey) {

        List<Map<String,String>> doclist = new ArrayList<>(100);
        try {
            long employeeId = employee.getEmployeeId();

            Map<String,String> docMap = new HashMap<String,String>();

//            String doc_count4 = getDocumentDateAmount(employeeId,"4",tenantKey);
//            String doc_name4 = "待查看文档数量";
//            String doc_pkey4 = "document.create.amount";
//            String doc_url4 =  jucailinCockpitUtils.getUfPropData(doc_pkey4,tenantKey);
//            docMap.put("count",doc_count4);
//            docMap.put("name",doc_name4);
//            docMap.put("url",doc_url4);
//            doclist.add(docMap);

            docMap = new HashMap<String,String>();
            String doc_count5 = getDocumentDateAmount5(employeeId,tenantKey);
            String doc_name5 = "我的积分";
            String doc_pkey5 = "document.mine.integral";
            String doc_url5 =  jucailinCockpitUtils.getUfPropData(doc_pkey5,tenantKey);
            docMap.put("count",doc_count5);
            docMap.put("name",doc_name5);
            docMap.put("url",doc_url5);
            doclist.add(docMap);

            docMap = new HashMap<String,String>();
            String doc_count6 = getDocumentDateAmount6(employeeId,tenantKey);
            String doc_name6 = "我的分享";
            String doc_pkey6 = "document.mine.share";
            String doc_url6 =  jucailinCockpitUtils.getUfPropData(doc_pkey6,tenantKey);
            docMap.put("count",doc_count6);
            docMap.put("name",doc_name6);
            docMap.put("url",doc_url6);
            doclist.add(docMap);


//            docMap = new HashMap<String,String>();
//            String doc_count7 = getDocumentDateAmount7(employeeId,tenantKey);
//            String doc_name7 = "下属知识";
//            String doc_pkey7 = "document.subordinate.create";
//            String doc_url7 =  jucailinCockpitUtils.getUfPropData(doc_pkey7,tenantKey);
//            docMap.put("count",doc_count7);
//            docMap.put("name",doc_name7);
//            docMap.put("url",doc_url7);
//            doclist.add(docMap);


            docMap = new HashMap<String,String>();
            String doc_count1 = getDocumentDateAmount1(employeeId,tenantKey);
            String doc_name1 = "我创建的文档";
            String doc_pkey1 = "document.create.amount";
            String doc_url1 =  jucailinCockpitUtils.getUfPropData(doc_pkey1,tenantKey);
            docMap.put("count",doc_count1);
            docMap.put("name",doc_name1);
            docMap.put("url",doc_url1);
            doclist.add(docMap);

//            docMap = new HashMap<String,String>();
//            String doc_count2 = getDocumentDateAmount(employeeId,"2",tenantKey);
//            String doc_name2 = "共享给我的文档";
//            String doc_pkey2 = "document.read.amount";
//            String doc_url2 =  jucailinCockpitUtils.getUfPropData(doc_pkey2,tenantKey);
//            docMap.put("count",doc_count2);
//            docMap.put("name",doc_name2);
//            docMap.put("url",doc_url2);
//            doclist.add(docMap);
//
//            docMap = new HashMap<String,String>();
//            String doc_count3 = getDocumentDateAmount(employeeId,"3",tenantKey);
//            String doc_name3 = "我订阅的文档";
//            String doc_pkey3 = "document.number.amount";
//            String doc_url3 =  jucailinCockpitUtils.getUfPropData(doc_pkey3,tenantKey);
//            docMap.put("count",doc_count3);
//            docMap.put("name",doc_name3);
//            docMap.put("url",doc_url3);
//            doclist.add(docMap);


            docMap = new HashMap<String,String>();
            String doc_count8 = getDocumentDateAmount8(employeeId,tenantKey);
            String doc_name8 = "我关注的文档";
            String doc_pkey8 = "document.mine.attention";
            String doc_url8 =  jucailinCockpitUtils.getUfPropData(doc_pkey8,tenantKey);
            docMap.put("count",doc_count8);
            docMap.put("name",doc_name8);
            docMap.put("url",doc_url8);
            doclist.add(docMap);

        } catch (Exception e) {
            e.printStackTrace();
            log.error("DevQuestCountCmd-Exception:" + e);
        }
        return doclist;
    }


//    public String getDocumentDateAmount(Long employeeId,String type,String tenantKey) {
//        String count = "0";
//        String groupId = "weaver-doc-service";
//        String sourceType = "LOGIC";
//
//        try {
//            if("1".equals(type)){
//
//                int doccreatecount = docClientService.countByCreator(employeeId);
//                count = doccreatecount + "";
//
//            }else if("2".equals(type)){
//
//                String sql =" select count(1) as cou " +
//                        "  from document " +
//                        "  where id in( select targetid from doc_documentlogs where log_operator <> ? and operatetype = 'read' and delete_type = 0 and tenant_key=? and doc_status = 2 ) \n" +
//                        "  and creator = ?\n" +
//                        "  and delete_type = 0\n" +
//                        "  and tenant_key= ? " ;
//
//                List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
//                SqlParamEntity sqlParamEntity = new SqlParamEntity();
//                sqlParamEntity.setParamType(SqlParamType.LONG);
//                sqlParamEntity.setValue(employeeId+"");
//                sqlparam.add(sqlParamEntity);
//
//                sqlParamEntity = new SqlParamEntity();
//                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
//                sqlParamEntity.setValue(tenantKey);
//                sqlparam.add(sqlParamEntity);
//
//                sqlParamEntity = new SqlParamEntity();
//                sqlParamEntity.setParamType(SqlParamType.LONG);
//                sqlParamEntity.setValue(employeeId+"");
//                sqlparam.add(sqlParamEntity);
//
//                sqlParamEntity = new SqlParamEntity();
//                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
//                sqlParamEntity.setValue(tenantKey);
//                sqlparam.add(sqlParamEntity);
//
//                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
//                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
//                if(recordList.size()>0){
//                    count = String.valueOf(recordList.get(0).get("cou"));
//                }
//            }else if("3".equals(type)){
//                String sql =" select count(1) as cou \n" +
//                        " from doc_documentlogs  \n" +
//                        " where targetid in( select id from document where creator = ? and delete_type = 0 and tenant_key=? and doc_status = 2) \n" +
//                        " and log_operator <> ? \n" +
//                        " and OPERATETYPE = 'read' \n" +
//                        " and delete_type = 0 \n" +
//                        " and tenant_key=? " ;
//
//                List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
//                SqlParamEntity sqlParamEntity = new SqlParamEntity();
//                sqlParamEntity.setParamType(SqlParamType.LONG);
//                sqlParamEntity.setValue(employeeId+"");
//                sqlparam.add(sqlParamEntity);
//
//                sqlParamEntity = new SqlParamEntity();
//                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
//                sqlParamEntity.setValue(tenantKey);
//                sqlparam.add(sqlParamEntity);
//
//                sqlParamEntity = new SqlParamEntity();
//                sqlParamEntity.setParamType(SqlParamType.LONG);
//                sqlParamEntity.setValue(employeeId+"");
//                sqlparam.add(sqlParamEntity);
//
//                sqlParamEntity = new SqlParamEntity();
//                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
//                sqlParamEntity.setValue(tenantKey);
//                sqlparam.add(sqlParamEntity);
//
//                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
//                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
//                if(recordList.size()>0){
//                    count = String.valueOf(recordList.get(0).get("cou"));
//                }
//            }else if("4".equals(type)){
//
//            }else if("5".equals(type)){
//
//            }else if("6".equals(type)){
//
//            }else if("7".equals(type)){
//
//            }else if("8".equals(type)){
//
//            }
//        } catch (Exception e) {
//            e.printStackTrace();
//            log.error("getDocCreateNum:" + e);
//        }
//
//        return count;
//    }

    /***
     *
     * @param employeeId
     * @param tenantKey
     * @return
     */
    public String getDocumentDateAmount1(Long employeeId,String tenantKey) {
        String count = "0";
        try {
            int doccreatecount = docClientService.countByCreator(employeeId);
            count = doccreatecount + "";
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDocCreateNum:" + e);
        }

        return count;
    }

    /***
     *
     * @param employeeId
     * @param tenantKey
     * @return
     */
    public String getDocumentDateAmount5(Long employeeId,String tenantKey) {
        String count = "0";

        String groupId = "weaver-ebuilder-form-service";
        String sourceType = "LOGIC";

        try {
            if(StringUtils.isNotBlank(employeeId+"")){
                String sql =" select sum(case when jfzj = 0 then jfz else 0-jfz end ) as zjf \n" +
                            " from uf_docPointDetail_112 " +
                            " where xm = ?"+
                            " and delete_type = 0" +
                            " and tenant_key = ?" ;

                List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
                SqlParamEntity sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
                if(recordList.size()>0){
                    count = String.valueOf(recordList.get(0).get("zjf"));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDocCreateNum:" + e);
        }
        return count;
    }

    /***
     *
     * @param employeeId
     * @param tenantKey
     * @return
     */
    public String getDocumentDateAmount6(Long employeeId,String tenantKey) {
        int count = 0;
        String count1 = "0";
        String count2 = "0";
        String groupId = "weaver-doc-service";
        String sourceType = "LOGIC";
        try {

            if(StringUtils.isNotBlank(employeeId+"")){
                String sql ="  select count(1) as cou\n" +
                            "  from doc_external_share \n" +
                            "  where creator = ? " +
                            "  and date_add(create_time, interval period_validity day) >= curdate()" +
                            "  and delete_type = 0" +
                            "  and tenant_key = ?" ;

                List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
                SqlParamEntity sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
                if(recordList.size()>0){
                    count1 = String.valueOf(recordList.get(0).get("cou"));
                }
            }

        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDocumentDateAmount6:" + e);
        }

        try {

            if(StringUtils.isNotBlank(employeeId+"")){
                String sql ="  select count(1) as cou " +
                            "  from doc_matter_forwardlogs " +
                            "  where creator = ? \n" +
                            "  and delete_type = 0 \n" +
                            "  and tenant_key = ?\n" +
                            "  and functionname = 'matter_forward'\n" +
                            "  and operatetype = 0 " +
                            "  and targetid not in(\n" +
                            "       select targetid " +
                            "       from doc_matter_forwardlogs " +
                            "       where creator = ?\n" +
                            "       and delete_type = 0 \n" +
                            "       and tenant_key = ?\n" +
                            "       and functionname = 'matter_forward'\n" +
                            "       and operatetype=1 \n" +
                            "   )" ;

                List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
                SqlParamEntity sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
                if(recordList.size()>0){
                    count2 = String.valueOf(recordList.get(0).get("cou"));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDocumentDateAmount6:" + e);
        }

        try{
            if(StringUtils.isNotBlank(count1)){
                int cou1 = Integer.parseInt(count1);
                count = count + cou1;
            }

            if(StringUtils.isNotBlank(count2)){
                int cou2 = Integer.parseInt(count2);
                count = count + cou2;
            }
        }catch (Exception e){
            e.printStackTrace();
            log.error("getDocumentDateAmount6:" + e);
        }

        return count+"";
    }


    public String getDocumentDateAmount7(Long employeeId,String tenantKey) {
        String count = "0";
        String groupId = "weaver-doc-service";
        String sourceType = "LOGIC";

        try {
            String sql =" select count(1) as cou from document where creator in(" +
                        "       select id from eteams.employee where superior = ? and status='normal' and delete_type=0 and tenant_key=? "+
                        " ) and delete_type = 0 and tenant_key=? and doc_status = 2" ;

            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.LONG);
            sqlParamEntity.setValue(employeeId+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                count = String.valueOf(recordList.get(0).get("cou"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDocCreateNum:" + e);
        }
        return count;
    }


    public String getDocumentDateAmount8(Long employeeId,String tenantKey) {
        String count = "0";
        String groupId = "weaver-doc-service";
        String sourceType = "LOGIC";

        try {
            String sql =" select count(1) as cou \n" +
                        " from my_watch  \n" +
                        " where watcher = ? \n" +
                        " and module='document' \n" +
                        " and delete_type = 0 \n" +
                        " and tenant_key=? " ;

            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.LONG);
            sqlParamEntity.setValue(employeeId+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                count = String.valueOf(recordList.get(0).get("cou"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDocCreateNum:" + e);
        }
        return count;
    }


    /***
     *
     * @param employee
     * @param tenantKey
     * @return
     */
    public List<Map<String,Object>> getRightViewData(SimpleEmployee employee,String tenantKey){
        List<Map<String,Object>> recordList = new ArrayList<Map<String,Object>>();
        String groupId = "weaver-doc-service";
        String sourceType = "LOGIC";
        Long employeeId = employee.getEmployeeId();
        log.error("employeeId:"+employeeId);
        try {
            String sql =" select p.folder,p.name,p.cou from ( \n" +
                        "   select w.folder,w.name,w.cou \n" +
                        "   from ( \n" +
                        "    select t.folder,l.name,count(1) as cou \n" +
                        "    from document t \n" +
                        "    inner join folder l on l.id = t.folder \n" +
                        "    where t.creator = ?  \n" +
                        "    and t.delete_type = 0 \n" +
                        "    and t.tenant_key= ? " +
                        "    and l.delete_type = 0 \n" +
                        "    and l.tenant_key= ? " +
                        "    group by t.folder,l.name \n" +
                        "  ) w  order by w.cou desc  \n" +
                        ") p limit 5 " ;
            log.error("sql:"+sql);

            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.LONG);
            sqlParamEntity.setValue(employeeId+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            recordList = databaseUtils.getDataSourceList(result);

        } catch (Exception e) {
            e.printStackTrace();
            log.error("getRightViewData:" + e);
        }
        return recordList;
    }





}
