package com.weaver.seconddev.cockpit.cmd;

import com.weaver.framework.rpc.annotation.RpcReference;
import com.weaver.teams.domain.hr.entity.FlowData;
import com.weaver.teams.domain.hr.entity.FlowDataListReq;
import com.weaver.teams.domain.hr.entity.FlowDataListRes;
import com.weaver.teams.domain.user.SimpleEmployee;
import com.weaver.workrelate.performance.api.rest.RemotePerformanceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.*;

/**
 * 月绩效信息
 *
 * @author wangj
 * @version 1.00版本
 * @Date 2024/3/8
 */
@Component
public class CockpitMonthPerformanceInfoCmd {
    //日志
    private final static Logger log = LoggerFactory.getLogger(CockpitMonthPerformanceInfoCmd.class);

    @RpcReference
    public RemotePerformanceService remotePerformanceService;

    public Map<String, Object> getDataInfo(SimpleEmployee simpleEmployee, Map<String, Object> paramMap,String tenantKey) {
        Map<String, Object> dataMap = new HashMap<String, Object>(100);
        String dateType = "month";
        try {
            long employeeId = simpleEmployee.getEmployeeId();
            List<Long> userList = new ArrayList<Long>();
            userList.add(employeeId);

            LocalDate currentDate = LocalDate.now();
            List<Map<String, Object>> scoreList = new ArrayList<Map<String, Object>>();
            Map<String, Object> scoreMap = null;
            for (int i = 0; i < 6; i++) {
                LocalDate lastLocalDate = currentDate.minus(i, ChronoUnit.MONTHS); // 当前日期减去6个月
                LocalDate startDay = lastLocalDate.with(TemporalAdjusters.firstDayOfMonth()); // 计算当前月份的首日
                LocalDate endDay = lastLocalDate.with(TemporalAdjusters.lastDayOfMonth()); // 获取本月最后一天

                log.error("startDay:"+startDay.toString());
                log.error("endDay:"+endDay.toString());

                String score =  getScore(userList,startDay.toString(), endDay.toString(),dateType,tenantKey);
                log.error("score:"+score);

                int month = lastLocalDate.getMonth().getValue();
                int year = lastLocalDate.getYear();

                String currentMonth = "" ;
                if(month < 10){
                    currentMonth = year+"-0"+month;
                }else{
                    currentMonth = year+"-"+month;
                }
                log.error("currentMonth:"+currentMonth);

                scoreMap = new HashMap<String, Object>();
                scoreMap.put("month",currentMonth);
                scoreMap.put("score",score);
                scoreList.add(scoreMap);
            }
            dataMap.put("datas",scoreList);

        } catch (Exception e) {
            e.printStackTrace();
            log.error("PortalPerformanceInfoCmd :" + e);
        }

        return dataMap;
    }


    public String getScore( List<Long> userList,String startDay, String endDay,String dateType,String tenantKey) {
        String scoreData = "0.00" ;
        try {
            FlowDataListReq flowDataListReq = new FlowDataListReq();
            flowDataListReq.setCurrent(1);
            flowDataListReq.setPageSize(20);
            flowDataListReq.setStartTime(startDay + "");
            flowDataListReq.setEndTime(endDay + "");
            flowDataListReq.setDateType(dateType);
            flowDataListReq.setTenantKey(tenantKey);

            flowDataListReq.setUserList(userList);
            FlowData flowData = remotePerformanceService.queryFlowData(flowDataListReq);
            log.error("getWholeYearKpi2-flowData:" + flowData);
            List<FlowDataListRes> flowDataList = flowData.getDataList();
            if (flowDataList.size() > 0) {
                for (int index = 0; index < flowDataList.size(); index++) {
                    FlowDataListRes flowDataListRes = flowDataList.get(index);
                    float score = flowDataListRes.getScore();
                    scoreData = score+"";
                }
            }
        } catch (Exception e) {
            log.error("getWholeYearKpi2-getScore-Exception:" + e.getMessage());
        }
        return scoreData;
    }

}
