package com.weaver.seconddev.cockpit.cmd;

import com.weaver.common.base.entity.result.WeaResult;
import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.ebuilder.datasource.api.enums.SqlParamType;
import com.weaver.framework.rpc.annotation.RpcReference;
import com.weaver.seconddev.cockpit.util.DatabaseUtils;
import com.weaver.seconddev.cockpit.util.JucailinCockpitUtils;
import com.weaver.teams.domain.hr.entity.FlowData;
import com.weaver.teams.domain.hr.entity.FlowDataListReq;
import com.weaver.teams.domain.hr.entity.FlowDataListRes;
import com.weaver.teams.domain.user.SimpleEmployee;
import com.weaver.teams.security.context.UserContext;
import com.weaver.workflow.common.constant.list.api.DateTimeConditionEnum;
import com.weaver.workflow.common.constant.list.listdimension.RequestListInitDimensionTabEnum;
import com.weaver.workflow.common.entity.list.api.RequestListConditionApiEntity;
import com.weaver.workflow.common.entity.list.api.publicapi.RequestCountInfoPAEntity;
import com.weaver.workflow.list.api.rest.publicapi.WflRequestListRest;
import com.weaver.workrelate.performance.api.rest.RemotePerformanceService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 本年工作成果信息(按类型)
 *
 * @author shil
 * @version 1.00版本
 * @Date 2024/5/22
 */
@Component
public class CockpitWorkAchievementsCmd {
    //日志
    private final static Logger log = LoggerFactory.getLogger(CockpitWorkAchievementsCmd.class);

    @RpcReference
    public RemotePerformanceService remotePerformanceService;

    @Autowired
    private JucailinCockpitUtils jucailinCockpitUtils;

    @Autowired
    private DatabaseUtils databaseUtils;

    @RpcReference(group = "workflow")
    WflRequestListRest wflRequestListRest;


    public Map<String, Object> getDataInfo(SimpleEmployee simpleEmployee, Map<String, Object> paramMap,String tenantKey) {

        Map<String, Object> recordMap = new HashMap<String, Object>();

        List<Map<String,String>> list = new ArrayList<Map<String,String>>(100);
        Map<String,String> oneMap = new HashMap<String,String>(100);
        Map<String,String> twoMap = new HashMap<String,String>(100);
        Map<String,String> threeMap = new HashMap<String,String>(100);
        Map<String,String> fourMap = new HashMap<String,String>(100);


        String startday = String.valueOf(paramMap.get("startday"));
        String endday = String.valueOf(paramMap.get("endday"));

        String one = "0";
        String oneword = "";
        String oneurl = "";

        String two = "0";
        String twoword = "";
        String twourl = "";

        String three = "0";
        String threeword = "";
        String threeurl = "";

        String four = "0";
        String fourword = "";
        String foururl = "";

        try {
            String employeeId = String.valueOf(simpleEmployee.getEmployeeId());

            //销售 0，项目 1,技术 2，其它 3
            String znlx  = getUserType(simpleEmployee,tenantKey);

            //销售 0，项目 1,技术 2，其它 3
            if("2".equals(znlx)){

                one = getQuestInfo(startday, endday, employeeId,tenantKey);
                oneword = "本年问题处理数";
                String pkey = "workdata.technology.wtclsl";
                oneurl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                two = getDaysInfo(startday, endday, employeeId,tenantKey);
                twoword = "本年开发人天";
                pkey = "workdata.technology.kfrt";
                twourl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                three = getCoWorkInfo(startday,endday, employeeId,tenantKey);
                threeword = "本年协作沟通数";
                pkey = "workdata.technology.gtxz";
                threeurl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                four = getWfInfo(simpleEmployee);
                fourword = "本年处理流程数";
                pkey = "workdata.technology.cllc";
                foururl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

            }else if("1".equals(znlx)){

                one = "0.00";
                oneword = "本年验收指标";
                String pkey = "workdata.project.yszb";
                oneurl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                //项目验收信息
                two = getYsInfo(employeeId,startday,endday);
                two = parseJe(two);
                log.error("four:"+four);
                twoword = "本年验收金额";
                pkey = "workdata.project.ysje";
                twourl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                three = "0.00";
                threeword = "本年指标达成率";
                pkey = "workdata.project.zbdcl";
                threeurl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                //项目收款信息
                four = getSkInfo(employeeId,startday,endday,tenantKey);
                log.error("four:"+four);
                four = parseJe(four);
                log.error("four2:"+four);
                fourword = "本年收款金额";
                pkey = "workdata.project.skje";
                foururl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

            }else if("3".equals(znlx)){

                one = getWfInfo(simpleEmployee);
                oneword = "本年处理流程数";
                String pkey = "workdata.other.cllc";
                oneurl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                two = getDocInfo(startday,endday,employeeId,tenantKey);
                twoword = "本年文档贡献度";
                pkey = "workdata.other.wdgx";
                twourl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                three = getDailyInfo(startday,endday,employeeId,tenantKey);
                threeword = "本年微博填写数";
                pkey = "workdata.other.wbtx";
                threeurl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                four = getCoWorkInfo(startday,endday,employeeId,tenantKey);
                fourword = "本年沟通协作数";
                pkey = "workdata.other.gtxz";
                foururl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

            }else{

                one = getWfInfo(simpleEmployee);
                oneword = "本年处理流程数";
                String pkey = "workdata.other.cllc";
                oneurl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                two = getDocInfo(startday,endday,employeeId,tenantKey);
                twoword = "本年文档贡献度";
                pkey = "workdata.other.wdgx";
                twourl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                three = getDailyInfo(startday,endday,employeeId,tenantKey);
                threeword = "本年微博填写数";
                pkey = "workdata.other.wbtx";
                threeurl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);

                four = getCoWorkInfo(startday,endday,employeeId,tenantKey);
                fourword = "本年沟通协作数";
                pkey = "workdata.other.gtxz";
                foururl =  jucailinCockpitUtils.getUfPropData(pkey,tenantKey);
            }

            oneMap.put("name",oneword);
            oneMap.put("value",one);
            oneMap.put("url",oneurl);

            log.error("1111");

            twoMap.put("name",twoword);
            twoMap.put("value",two);
            twoMap.put("url",twourl);

            log.error("2222");

            threeMap.put("name",threeword);
            threeMap.put("value",three);
            threeMap.put("url",threeurl);

            log.error("3333");

            fourMap.put("name",fourword);
            fourMap.put("value",four);
            fourMap.put("url",foururl);

            log.error("4444");

            list.add(oneMap);
            list.add(twoMap);
            list.add(threeMap);
            list.add(fourMap);
            
            recordMap.put("datas",list);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("CockpitWorkAchievementsCmd :" + e);
        }
        return recordMap;
    }


    public String getScore( List<Long> userList,String startDay, String endDay,String dateType,String tenantKey) {
        String scoreData = "0.00" ;
        try {
            FlowDataListReq flowDataListReq = new FlowDataListReq();
            flowDataListReq.setCurrent(1);
            flowDataListReq.setPageSize(20);
            flowDataListReq.setStartTime(startDay + "");
            flowDataListReq.setEndTime(endDay + "");
            flowDataListReq.setDateType(dateType);
            flowDataListReq.setTenantKey(tenantKey);

            flowDataListReq.setUserList(userList);
            FlowData flowData = remotePerformanceService.queryFlowData(flowDataListReq);
            log.error("getWholeYearKpi2-flowData:" + flowData);
            List<FlowDataListRes> flowDataList = flowData.getDataList();
            if (flowDataList.size() > 0) {
                for (int index = 0; index < flowDataList.size(); index++) {
                    FlowDataListRes flowDataListRes = flowDataList.get(index);
                    float score = flowDataListRes.getScore();
                    scoreData = score+"";
                }
            }
        } catch (Exception e) {
            log.error("getWholeYearKpi2-getScore-Exception:" + e.getMessage());
        }
        return scoreData;
    }

    /***
     *
     * @param startday
     * @param endday
     * @param employeeId
     * @param tenantKey
     * @return
     */
    public String getQuestInfo(String startday, String endday, String employeeId,String tenantKey) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String quecou = "0";

        String groupId = "weaver-workflow-report-serviceworkflowreport";
        String sourceType = "LOGIC";

        try {

            Calendar calendar = Calendar.getInstance();
            int year = calendar.get(Calendar.YEAR);
            if (StringUtils.isBlank(startday)) {
                startday = year + "-01-01";
            }
            if (StringUtils.isBlank(endday)) {
                endday = sdf.format(new Date());
            }

            if (StringUtils.isBlank(employeeId)) {
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }

            if (StringUtils.isNotBlank(employeeId)) {
                String dataSql =" select ifnull(sum(d.integer2),0) as devcou \n" +
                                " from wfc_form_data l\n" +
                                " inner join fto_62 d on d.id = l.dataid\n" +
                                " inner join wfc_requestbase p on p.requestid = l.requestid\n" +
                                " where d.multiresource5=?\n" +
                                " and d.date2 >=?" +
                                " and d.date2 <=?" +
                                " and d.tenant_key=?\n" +
                                " and d.is_delete='0'\n" +
                                " and l.tenant_key=?\n" +
                                " and l.delete_type=0\n" +
                                " and p.tenant_key=?\n" +
                                " and p.delete_type=0\n" +
                                " order by d.id desc ";

                log.info("DevQuestCountCmd dataSql:" + dataSql);

                List<String> strlist = new ArrayList<>(100);
                strlist.add(employeeId);
                strlist.add(startday);
                strlist.add(endday);
                strlist.add(tenantKey);
                strlist.add(tenantKey);
                strlist.add(tenantKey);

                List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);

                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, dataSql,sqlparam);
                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
                if(recordList.size()>0){
                    quecou = String.valueOf(recordList.get(0).get("devcou"));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("DevQuestCountCmd-Exception:" + e);
        }
        return quecou;
    }


    /**
     * 获取人员类型
     * @param employee
     * @return
     */
    public String getUserType(SimpleEmployee employee,String tenantkey) {
        String sourceType = "LOGIC";
        String userType = "3";
        try{
            long formdata = employee.getFormdata();
            String pkey = "field69.groupId";
            String groupId =  jucailinCockpitUtils.getUfPropData(pkey,tenantkey);
            log.error("groupId:"+groupId);

            String field69 = "";
            String sql =" select field69 from eteams.Hrmemployeedefined where form_data_id = ?" ;
            log.error("getSelfServiceData--sql:"+sql);

            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.LONG);
            sqlParamEntity.setValue(formdata+"");
            sqlparam.add(sqlParamEntity);
            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                // 销售 0，项目 1,技术 2，其它 3
                field69 = String.valueOf(recordList.get(0).get("field69"));
            }
            log.error("field69:"+field69);
            if(StringUtils.isNotBlank(field69)){
                switch (field69){
                    case "1":
                        userType = "0";
                        break;
                    case "2":
                        userType = "1";
                        break;
                    case "3":
                        userType = "2";
                        break;
                    default:
                        userType = "3";
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            log.error("getUserType-e:"+e);
        }
        return userType;
    }

    /***
     *
     * @param startday
     * @param endday
     * @param employeeId
     * @return
     */
    public String getDaysInfo(String startday, String endday, String employeeId,String tenantKey) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        String devcou = "0";

        String groupId = "weaver-workflow-report-serviceworkflowreport";
        String sourceType = "LOGIC";

        try {
            Calendar calendar = Calendar.getInstance();
            int year = calendar.get(Calendar.YEAR);

            if (StringUtils.isBlank(startday)) {
                startday = year + "-01-01";
            }
            if (StringUtils.isBlank(endday)) {
                endday = sdf.format(new Date());
            }
            if (StringUtils.isBlank(employeeId)) {
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }

            if (StringUtils.isNotBlank(employeeId)) {
                String dataSql =" select count(1) as devcou \n" +
                                " from wfc_form_data l\n" +
                                " inner join fto_83 d on d.id = l.dataid\n" +
                                " inner join wfc_requestbase p on p.requestid = l.requestid\n" +
                                " where d.mutiresource= ?\n" +
                                " and d.xsjfrq >= ?" +
                                " and d.xsjfrq <= ?" +
                                " and d.tenant_key= ?\n" +
                                " and d.is_delete='0'\n" +
                                " and l.tenant_key=?\n" +
                                " and l.delete_type=0\n" +
                                " and p.tenant_key=?\n" +
                                " and p.delete_type=0\n" +
                                " order by d.id desc ";

                List<String> strlist = new ArrayList<>(100);
                strlist.add(employeeId);
                strlist.add(startday);
                strlist.add(endday);
                strlist.add(tenantKey);
                strlist.add(tenantKey);
                strlist.add(tenantKey);
                List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);
                log.info("DevDaysCountCmd sql:" + dataSql);
                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, dataSql,sqlparam);
                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
                if(recordList.size()>0){
                    devcou = String.valueOf(recordList.get(0).get("devcou"));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDaysInfo-Exception:" + e);
        }
        return devcou;
    }

    /**
     * 本年沟通协作数
     *
     * @param
     * @return
     */
    public String getCoWorkInfo(String startday, String endday, String employeeId,String tenantKey) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        String groupId = "weaver-cowork-service";
        String sourceType = "LOGIC";
        String count = "0";
        try {

            Calendar calendar = Calendar.getInstance();
            int year = calendar.get(Calendar.YEAR);

            if (StringUtils.isBlank(startday)) {
                startday = year + "-01-01";
            }
            if (StringUtils.isBlank(endday)) {
                endday = sdf.format(new Date());
            }
            if (StringUtils.isBlank(employeeId)) {
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }

            String sql =" select count(1) as count from cw_content " +
                        " where CREATOR=?\n" +
                        " and left(create_time,10)>=? " +
                        " and left(create_time,10) <= ?" +
                        " and tenant_key = ?" +
                        " and delete_type = 0 ";

            log.error("getInteractInfo sql:" + sql);

            List<String> strlist = new ArrayList<>(100);
            strlist.add(employeeId);
            strlist.add(startday);
            strlist.add(endday);
            strlist.add(tenantKey);

            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);
            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql, sqlparam);
            List<Map<String, Object>> recordList = databaseUtils.getDataSourceList(result);
            if (recordList.size() > 0) {
                count = String.valueOf(recordList.get(0).get("count"));
            }
        }catch (Exception e){
            e.printStackTrace();
            log.error("getCoWorkInfo-Exception:"+e.getMessage());
        }
        return count;
    }


    /**
     * 本年流程处理数量
     *
     * @param
     * @return
     */
    public String getWfInfo(SimpleEmployee employee) {
        String wfcount = "0";
        try{
            RequestListConditionApiEntity conditionEntity = new RequestListConditionApiEntity();
            conditionEntity.setOperatedate(DateTimeConditionEnum.CURRENT_YEAR);
            WeaResult<RequestCountInfoPAEntity> requestCountInfo = wflRequestListRest.getRequestCountByTabId(employee, RequestListInitDimensionTabEnum.DONE_FLOWALL.getTabid(), conditionEntity);
            int code = requestCountInfo.getCode();
            if(code == 200){
                RequestCountInfoPAEntity requestCountInfoPAEntity = requestCountInfo.getData();
                Long count = requestCountInfoPAEntity.getCount();
                wfcount = count+"" ;
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getWfInfo-Exception:" + e);
        }
        return wfcount;
    }


    /***
     *
     * @param employeeId
     * @param startday
     * @param endday
     * @return
     */
    public String getSkInfo(String employeeId,String startday,String endday,String tenantKey) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        String groupId = "weaver-ebuilder-form-service";
        String sourceType = "LOGIC";

        String skje = "0.00";
        try {

            Calendar calendar = Calendar.getInstance();
            int year = calendar.get(Calendar.YEAR);

            if(StringUtils.isBlank(startday)){
                startday = year+"-01-01";
            }
            if(StringUtils.isBlank(endday)){
                endday = sdf.format(new Date());
            }

            if(StringUtils.isBlank(employeeId)){
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }


            String sql =" SELECT \n" +
                        "   CONVERT(ifnull(sum(cast(t1.skje as decimal(18,2))),0),CHAR) as skje,\n" +  //收款金额
                        "   CONVERT(ifnull(sum(cast(t1.xmskje as decimal(18,2))),0),CHAR) as xmskje,\n" + // 项目收款金额
                        "   CONVERT(ifnull(sum(cast(t1.xsyxje as decimal(18,2))),0),CHAR) as xsyxje," + // 销售有效金额
                        "   CONVERT(ifnull(sum(cast(t1.xsje as decimal(18,2))),0),CHAR) as xsje," + // 销售金额
                        "   CONVERT(ifnull(sum(cast(t1.xmyxje as decimal(18,2))),0),CHAR) as xmyxje," + //项目有效金额
                        "   CONVERT(ifnull(sum(cast(t1.xsjxhk as decimal(18,2))),0),CHAR) as xsjxhk," + //有效绩效回款
                        "   CONVERT(ifnull(sum(cast(t1.sumsk as decimal(18,2))),0),CHAR) as sumsk," + //总收款金额
                        "   CONVERT(ifnull(sum(cast(t1.sjjsk as decimal(18,2))),0),CHAR) as sjjsk," + //实际结算款
                        "   CONVERT(ifnull(sum(cast(t1.skjs as decimal(18,2))),0),CHAR) as skjs " +  //收款结算
                        " FROM uf_jxhs_skmx t1\n" +
                        " LEFT JOIN uf_jxhs_zxhtzb t2 ON t1.hth = t2.hth AND t2.hth != '' AND t2.hth IS NOT NULL \n" +
                        " LEFT JOIN ebdf_physical_data epd on t1.id = epd.form_data_id  and epd.delete_type = 0 \n" +
                        " WHERE t1.skje != 0 \n" +
                        " and t1.skje IS NOT NULL " +
                        " and t1.xshsny >= ? and t1.xshsny <=? "+
                        " and (" +
                        "   ? IN ( 2, 27, 166, 10285, 11661, 300, 876, 3593, 1416, 1647, 1844,3044480226941419013,794508876297846787 ) \n" +
                        "   or cast(t1.xsjl AS decimal(25,0)) in( \n" +
                        "       SELECT cid AS id FROM eteams.emp_link WHERE pid = ?  and RELATION in ('othersenior','senior') \n" +
                        "       UNION  " +
                        "       SELECT id FROM eteams.employee WHERE id =  ? " +
                        "   )\n" +
                        "   or cast(t1.kfry AS decimal(25,0)) in(\n" +
                        "       SELECT cid AS id FROM eteams.emp_link WHERE pid =  ?  and RELATION in ('othersenior','senior') \n" +
                        "       UNION  " +
                        "       SELECT id FROM eteams.employee WHERE id =  ? " +
                        "   )\n" +
                        "   or cast(t1.qdjl AS decimal(25,0)) in(\n" +
                        "       SELECT cid AS id FROM eteams.emp_link WHERE pid =  ?  and RELATION in ('othersenior','senior') \n" +
                        "       UNION  " +
                        "       SELECT id FROM eteams.employee WHERE id =  ? " +
                        "   )\n" +
                        "   or cast(t1.xmjl AS decimal(25,0)) in(\n" +
                        "       SELECT cid AS id FROM eteams.emp_link WHERE pid =  ? and RELATION in ('othersenior','senior') \n" +
                        "       UNION  " +
                        "       SELECT id FROM eteams.employee WHERE id =  ? " +
                        "   )\n" +
                        " )\n" +
                        " order by t1.id " ;

            log.error("PorojectReceiptsInfoCmd sql:"+sql);
            List<String> strlist = new ArrayList<>(100);
            strlist.add(startday);
            strlist.add(endday);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);
            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            log.error("recordList:"+recordList.size());

            if(!CollectionUtils.isEmpty(recordList)){
                skje = String.valueOf(recordList.get(0).get("skje"));
            }
            log.error("skje:"+skje);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("PorojectReceiptsInfoCmd Exception:"+e);
        }
        return skje;
    }


    public String getYsInfo(String employeeId,String startday,String endday){

        String groupId = "weaver-ebuilder-form-service";
        String sourceType = "LOGIC";

        String htzje = "0.00" ;

        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        try {
            Calendar calendar = Calendar.getInstance();
            int year = calendar.get(Calendar.YEAR);

            if(StringUtils.isBlank(startday)){
                startday = year+"-01-01";
            }
            if(StringUtils.isBlank(endday)){
                endday = sdf.format(new Date());
            }
            if(StringUtils.isBlank(employeeId)){
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }

            String sql =" select CONVERT(ifnull(sum(cast(t1.htzje as decimal(18,2))),0),CHAR) as htzje," +
                        "        CONVERT(ifnull(sum(cast(t1.htyxje as decimal(18,2))),0),CHAR) as htyxje," +
                        "        CONVERT(ifnull(sum(cast(t1.sumsk as decimal(18,2))),0),CHAR) as sumsk " +
                        " from uf_jxhs_xmys t1 \n" +
                        " where t1.ysrq>=? and t1.ysrq<=? " +
                        " and ( t1.qyjl = ? "+
                        "       or t1.qyjl in (select cid  from eteams.EMP_LINK where RELATION in ('othersenior','senior') and pid= ?)\n" +
                        "       or t1.ysry = ?\n" +
                        "       or t1.ysry in (select cid  from eteams.EMP_LINK where RELATION in ('othersenior','senior') and pid= ?)\n" +
                        "       or t1.dqfzr = ?\n" +
                        "       or t1.dqfzr in (select cid  from eteams.EMP_LINK where RELATION in ('othersenior','senior') and pid= ?)\n" +
                        "       or t1.tdjl = ?\n" +
                        "       or t1.tdjl in (select cid  from eteams.EMP_LINK where RELATION in ('othersenior','senior') and pid= ?)\n" +
                        "       or t1.tdfzr = ?\n" +
                        "       or t1.tdfzr in (select cid  from eteams.EMP_LINK where RELATION in ('othersenior','senior') and pid= ?)\n" +
                        "       or t1.jgfzr = ?\n" +
                        "       or t1.jgfzr in (select cid  from eteams.EMP_LINK where RELATION in ('othersenior','senior') and pid= ?)\n" +
                        "       or t1.dqzj = ?\n" +
                        "       or t1.dqzj in (select cid  from eteams.EMP_LINK where RELATION in ('othersenior','senior') and pid= ?) " +
                        " ) " ;

            List<String> strlist = new ArrayList<>(100);
            strlist.add(startday);
            strlist.add(endday);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);
            strlist.add(employeeId);

            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);
            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);

            if(!recordList.isEmpty() && recordList.size()>0){
                htzje = String.valueOf(recordList.get(0).get("htzje"));
            }

        } catch (Exception e) {
            e.printStackTrace();
            log.error("PorojectAcceptanceInfoCmd:"+e.getMessage());
        }
        return htzje;
    }


    /**
     * 本年文档创建贡献数
     * @param
     * @return
     */
    public String getDocInfo(String startday,String endday,String employeeId,String tenantKey) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        String groupId = "weaver-doc-service";
        String sourceType = "LOGIC";

        String count = "0";
        try{
            Calendar calendar = Calendar.getInstance();
            int year = calendar.get(Calendar.YEAR);

            if(StringUtils.isBlank(startday)){
                startday = year+"-01-01";
            }
            if(StringUtils.isBlank(endday)){
                endday = sdf.format(new Date());
            }

            if(StringUtils.isBlank(employeeId)){
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }

            String sql = " select count(1) as count " +
                         " from document " +
                         " where CREATOR=?\n" +
                         " and left(create_time,10)>=? " +
                         " and left(create_time,10) <= ?" ;

            log.error("PortalDocCreateCountCmd sql:"+sql);
            List<String> strlist = new ArrayList<>(100);
            strlist.add(employeeId);
            strlist.add(startday);
            strlist.add(endday);

            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                count = String.valueOf(recordList.get(0).get("count"));
            }
        }catch (Exception e){
            e.printStackTrace();
            log.error("getDocInfo-e:"+e);
        }
        return count;
    }


    public String getDailyInfo(String startday,String endday,String employeeId,String tenantKey) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        String groupId = "weaver-blog-service";
        String sourceType = "LOGIC";
        String count = "0";
        try{
            Calendar calendar = Calendar.getInstance();
            int year = calendar.get(Calendar.YEAR);

            if(StringUtils.isBlank(startday)){
                startday = year+"-01-01";
            }
            if(StringUtils.isBlank(endday)){
                endday = sdf.format(new Date());
            }

            if(StringUtils.isBlank(employeeId)){
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }

            String sql =" select count(1) as count\n" +
                    "    from blog " +
                    "    where creator=?\n" +
                    "    and left(BLOG_TIME,10) >=?\n" +
                    "    and left(BLOG_TIME,10) <=?\n" +
                    "    and type='blog'\n" +
                    "    and TENANT_KEY = ?\n" +
                    "    and MODULE = 'blog'\n" +
                    "    and DELETE_TYPE = 0\n" +
                    "    and CONTENT is not null" ;

            List<String> strlist = new ArrayList<>(100);
            strlist.add(employeeId);
            strlist.add(startday);
            strlist.add(endday);
            strlist.add(tenantKey);

            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                count = String.valueOf(recordList.get(0).get("count"));
            }

        }catch (Exception e){
            e.printStackTrace();
            log.error("getDailyInfo-e:"+e);
        }
        return count;
    }



    public static String parseJe(String value){
        if("".equals(value)){
            return "0.00";
        }
        BigDecimal dividend = new BigDecimal(value); // 被除数
        BigDecimal divisor = new BigDecimal("10000");  // 除数

        // 设置小数点后两位，并进行四舍五入
        BigDecimal result = dividend.divide(divisor, 2, RoundingMode.HALF_UP);

        return result.toString();
    }

}
