package com.weaver.seconddev.cockpit.cmd;

import com.weaver.calendar.service.RemoteAgendaService;
import com.weaver.common.base.entity.result.WeaResult;
import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.ebuilder.datasource.api.enums.SqlParamType;
import com.weaver.framework.rpc.annotation.RpcReference;
import com.weaver.seconddev.cockpit.util.DatabaseUtils;
import com.weaver.seconddev.cockpit.util.JucailinCockpitUtils;
import com.weaver.teams.client.doc.remote.DocClientService;
import com.weaver.teams.domain.user.SimpleEmployee;
import com.weaver.workflow.common.constant.list.listdimension.RequestListInitDimensionTabEnum;
import com.weaver.workflow.common.entity.list.api.RequestListConditionApiEntity;
import com.weaver.workflow.common.entity.list.api.publicapi.RequestCountInfoPAEntity;
import com.weaver.workflow.list.api.rest.publicapi.WflRequestListRest;
import com.weaver.workrelate.performance.api.rest.RemotePerformanceService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import java.util.*;

/**
 * 流程、日程、协助、文档 4个类型数据
 *
 * @author shil
 * @version 1.00版本
 * @Date 2024/5/22
 */
@Component
public class CockpitWorkDate4TypeCmd {
    //日志
    private final static Logger log = LoggerFactory.getLogger(CockpitWorkDate4TypeCmd.class);

    @RpcReference
    public RemotePerformanceService remotePerformanceService;

    @Autowired
    private JucailinCockpitUtils jucailinCockpitUtils;

    @Autowired
    private DatabaseUtils databaseUtils;

    @RpcReference(group = "workflow")
    WflRequestListRest wflRequestListRest;

    @RpcReference
    DocClientService docClientService;

    @RpcReference
    RemoteAgendaService remoteAgendaService;

    public Map<String, Object> getDataInfo(SimpleEmployee employee, Map<String, Object> paramMap,String tenantKey) {

        Map<String, Object> recordMap = new HashMap<String, Object>();

        List<Map<String,String>> wfList = new ArrayList<Map<String,String>>();
        List<Map<String,String>> docList = new ArrayList<Map<String,String>>();
        List<Map<String,String>> coworkList = new ArrayList<Map<String,String>>();
        List<Map<String,String>> calendarList = new ArrayList<Map<String,String>>();

        try {
            wfList = getWorkflowDate(employee,tenantKey);

            docList = getDocumentDate(employee,tenantKey);

            coworkList = getCoWorkDate(employee,tenantKey);

            calendarList = getCalendarDate(employee,tenantKey);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("PortalPerformanceInfoCmd :" + e);
        }

        recordMap.put("wfdata",wfList);
        recordMap.put("docdata",docList);
        recordMap.put("coworkdata",coworkList);
        recordMap.put("calendardata",calendarList);

        return recordMap;
    }

    /***
     *
     * @return
     */
    public List<Map<String,String>> getWorkflowDate(SimpleEmployee employee,String tenantKey) {

        List<Map<String,String>> wflist = new ArrayList<>(100);
        try {
            Map<String,String> wfMap = new HashMap<String,String>();
            String wf_count1 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.TODO_FLOWALL.getTabid());
            String wf_name1 = "待办流程数";
            String wf_pkey1 = "workflow.todo.amount";
            String wf_url1 =  jucailinCockpitUtils.getUfPropData(wf_pkey1,tenantKey);
            wfMap.put("count",wf_count1);
            wfMap.put("name",wf_name1);
            wfMap.put("url",wf_url1);
            wflist.add(wfMap);

            wfMap = new HashMap<String,String>();
            String wf_count2 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.MINE_FLOWALL.getTabid());
            String wf_name2 = "我发起的流程";
            String wf_pkey2 = "workflow.mine.amount";
            String wf_url2 =  jucailinCockpitUtils.getUfPropData(wf_pkey2,tenantKey);
            wfMap.put("count",wf_count2);
            wfMap.put("name",wf_name2);
            wfMap.put("url",wf_url2);
            wflist.add(wfMap);

            wfMap = new HashMap<String,String>();
            String wf_count3 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.DONE_FLOWALL.getTabid());
            String wf_name3 = "已办流程";
            String wf_pkey3 = "workflow.done.amount";
            String wf_url3 =  jucailinCockpitUtils.getUfPropData(wf_pkey3,tenantKey);
            wfMap.put("count",wf_count3);
            wfMap.put("name",wf_name3);
            wfMap.put("url",wf_url3);
            wflist.add(wfMap);

        } catch (Exception e) {
            e.printStackTrace();
            log.error("DevQuestCountCmd-Exception:" + e);
        }
        return wflist;
    }

    /***
     *
     * @param employee
     * @param tenantKey
     * @return
     */
    public List<Map<String,String>> getDocumentDate(SimpleEmployee employee,String tenantKey) {

        List<Map<String,String>> doclist = new ArrayList<>(100);
        try {
            long employeeId = employee.getEmployeeId();

            Map<String,String> docMap = new HashMap<String,String>();
            String doc_count1 = getDocumentDateAmount(employeeId,"1",tenantKey);
            String doc_name1 = "创建文档";
            String doc_pkey1 = "document.create.amount";
            String doc_url1 =  jucailinCockpitUtils.getUfPropData(doc_pkey1,tenantKey);
            docMap.put("count",doc_count1);
            docMap.put("name",doc_name1);
            docMap.put("url",doc_url1);
            doclist.add(docMap);

            docMap = new HashMap<String,String>();
            String doc_count2 = getDocumentDateAmount(employeeId,"2",tenantKey);
            String doc_name2 = "查阅文档";
            String doc_pkey2 = "document.read.amount";
            String doc_url2 =  jucailinCockpitUtils.getUfPropData(doc_pkey2,tenantKey);
            docMap.put("count",doc_count2);
            docMap.put("name",doc_name2);
            docMap.put("url",doc_url2);
            doclist.add(docMap);

            docMap = new HashMap<String,String>();
            String doc_count3 = getDocumentDateAmount(employeeId,"3",tenantKey);
            String doc_name3 = "查阅次数";
            String doc_pkey3 = "document.number.amount";
            String doc_url3 =  jucailinCockpitUtils.getUfPropData(doc_pkey3,tenantKey);
            docMap.put("count",doc_count3);
            docMap.put("name",doc_name3);
            docMap.put("url",doc_url3);
            doclist.add(docMap);

        } catch (Exception e) {
            e.printStackTrace();
            log.error("DevQuestCountCmd-Exception:" + e);
        }
        return doclist;
    }

    /***
     *
     * @param employee
     * @param tenantKey
     * @return
     */
    public List<Map<String,String>> getCoWorkDate(SimpleEmployee employee,String tenantKey) {

        List<Map<String,String>> datalist = new ArrayList<>(100);
        try {
            long employeeId = employee.getEmployeeId();

            Map<String,String> coworkMap = new HashMap<String,String>();
            String cowork_count = getCoWorkDateAmount(employeeId,"1",tenantKey);
            String cowork_name = "协作区发表";
            String cowork_pkey = "cowork.create.amount";
            String cowork_url =  jucailinCockpitUtils.getUfPropData(cowork_pkey,tenantKey);
            coworkMap.put("count",cowork_count);
            coworkMap.put("name",cowork_name);
            coworkMap.put("url",cowork_url);
            datalist.add(coworkMap);

            Map<String,String> imMap = new HashMap<String,String>();
            String im_count2 = getCoWorkDateAmount(employeeId,"2",tenantKey);
            String im_name2 = "参与群聊";
            String im_pkey2 = "im.group.amount";
            String im_url2 =  jucailinCockpitUtils.getUfPropData(im_pkey2,tenantKey);
            imMap.put("count",im_count2);
            imMap.put("name",im_name2);
            imMap.put("url",im_url2);
            datalist.add(imMap);

            imMap = new HashMap<String,String>();
            String im_count3 = getCoWorkDateAmount(employeeId,"3",tenantKey);
            String im_name3 = "平均每日沟通人数";
            String im_pkey3 = "im.chatday.amount";
            String im_url3 =  jucailinCockpitUtils.getUfPropData(im_pkey3,tenantKey);
            imMap.put("count",im_count3);
            imMap.put("name",im_name3);
            imMap.put("url",im_url3);
            datalist.add(imMap);

        } catch (Exception e) {
            e.printStackTrace();
            log.error("DevQuestCountCmd-Exception:" + e);
        }
        return datalist;
    }


    public List<Map<String,String>> getCalendarDate(SimpleEmployee employee,String tenantKey) {

        List<Map<String,String>> datalist = new ArrayList<>(100);
        try {
            long employeeId = employee.getEmployeeId();

            Map<String,String> calendarMap = new HashMap<String,String>();
            String calendar_count = getCalendarDateAmount(employeeId,"1",tenantKey);
            String calendar_name = "日程";
            String calendar_pkey = "calendar.create.amount";
            String calendar_url =  jucailinCockpitUtils.getUfPropData(calendar_pkey,tenantKey);
            calendarMap.put("count",calendar_count);
            calendarMap.put("name",calendar_name);
            calendarMap.put("url",calendar_url);
            datalist.add(calendarMap);

            calendarMap = new HashMap<String,String>();
            String calendar_count2 = getCalendarDateAmount(employeeId,"2",tenantKey);
            String calendar_name2 = "我的任务";
            String calendar_pkey2 = "task.mine.amount";
            String calendar_url2 =  jucailinCockpitUtils.getUfPropData(calendar_pkey2,tenantKey);
            calendarMap.put("count",calendar_count2);
            calendarMap.put("name",calendar_name2);
            calendarMap.put("url",calendar_url2);
            datalist.add(calendarMap);

            calendarMap = new HashMap<String,String>();
            String calendar_count3 = getCalendarDateAmount(employeeId,"3",tenantKey);
            String calendar_name3 = "关注任务";
            String calendar_pkey3 = "task.watch.amount";
            String calendar_url3 =  jucailinCockpitUtils.getUfPropData(calendar_pkey3,tenantKey);
            calendarMap.put("count",calendar_count3);
            calendarMap.put("name",calendar_name3);
            calendarMap.put("url",calendar_url3);
            datalist.add(calendarMap);

        } catch (Exception e) {
            e.printStackTrace();
            log.error("DevQuestCountCmd-Exception:" + e);
        }
        return datalist;
    }


    /**
     * 获取文档数量
     *
     * @param employeeId
     * @return
     */

    public String getCoWorkDateAmount(Long employeeId,String type,String tenantKey) {
        String count = "0";
        if("1".equals(type)){

            String groupId = "weaver-cowork-service";
            String sourceType = "LOGIC";

            String sql =" select count(1) as cou " +
                        " from cw_content " +
                        " where creator=?\n" +
                        "  and delete_type = 0\n" +
                        "  and tenant_key= ? " ;

            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.LONG);
            sqlParamEntity.setValue(employeeId+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                count = String.valueOf(recordList.get(0).get("cou"));
            }
        }else if("2".equals(type)){

            String groupId = "weaver-ebuilder-form-service";
            String sourceType = "LOGIC";

            String sql =" select amount " +
                    "  from uf_nj_pjmrdlhhsl " +
                    "  where ry = ?\n" +
                    "  and delete_type = 0\n" +
                    "  and tenant_key= ? " ;

            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.LONG);
            sqlParamEntity.setValue(employeeId+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                count = String.valueOf(recordList.get(0).get("amount"));
            }
        }else if("3".equals(type)){

            String groupId = "weaver-ebuilder-form-service";
            String sourceType = "LOGIC";

            String sql =" select amount " +
                        "  from uf_nj_cyqltjb " +
                        "  where ry = ?\n" +
                        "  and delete_type = 0\n" +
                        "  and tenant_key= ? " ;

            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.LONG);
            sqlParamEntity.setValue(employeeId+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                count = String.valueOf(recordList.get(0).get("amount"));
            }
        }
        return count;
    }

    public String getDocumentDateAmount(Long employeeId,String type,String tenantKey) {
        String count = "0";
        String groupId = "weaver-doc-service";
        String sourceType = "LOGIC";

        try {
            if("1".equals(type)){

                int doccreatecount = docClientService.countByCreator(employeeId);
                count = doccreatecount + "";

            }else if("2".equals(type)){

                String sql =" select count(1) as cou " +
                        "  from document " +
                        "  where id in( select targetid from doc_documentlogs where log_operator <> ? and operatetype = 'read' and delete_type = 0 and tenant_key=? and doc_status = 2 ) \n" +
                        "  and creator = ?\n" +
                        "  and delete_type = 0\n" +
                        "  and tenant_key= ? " ;

                List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
                SqlParamEntity sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
                if(recordList.size()>0){
                    count = String.valueOf(recordList.get(0).get("cou"));
                }
            }else if("3".equals(type)){
                String sql =" select count(1) as cou \n" +
                        " from doc_documentlogs  \n" +
                        " where targetid in( select id from document where creator = ? and delete_type = 0 and tenant_key=? and doc_status = 2) \n" +
                        " and log_operator <> ? \n" +
                        " and OPERATETYPE = 'read' \n" +
                        " and delete_type = 0 \n" +
                        " and tenant_key=? " ;

                List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
                SqlParamEntity sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
                if(recordList.size()>0){
                    count = String.valueOf(recordList.get(0).get("cou"));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDocCreateNum:" + e);
        }

        return count;
    }



    public String getCalendarDateAmount(Long employeeId,String type,String tenantKey) {
        String count = "0";
        String groupId = "weaver-doc-service";
        String sourceType = "LOGIC";

        try {
            if("1".equals(type)){

                int clendarcount = remoteAgendaService.countCalendar(employeeId,tenantKey);
                count = clendarcount + "";

            }else if("2".equals(type)){

                String sql =" select count(1) as cou " +
                            " from task " +
                            " where creator = ? \n" +
                            "  and STATUS = 'todo'\n" +
                            "  and delete_type = 0\n" +
                            "  and tenant_key= ? " ;

                List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
                SqlParamEntity sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
                if(recordList.size()>0){
                    count = String.valueOf(recordList.get(0).get("cou"));
                }
            }else if("3".equals(type)){
                String sql =" select count(1) as cou \n" +
                            " from my_watch  \n" +
                            " where watcher = ? \n" +
                            " and module='task' \n" +
                            " and delete_type = 0 \n" +
                            " and tenant_key=? " ;

                List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
                SqlParamEntity sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.LONG);
                sqlParamEntity.setValue(employeeId+"");
                sqlparam.add(sqlParamEntity);

                sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(tenantKey);
                sqlparam.add(sqlParamEntity);

                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
                if(recordList.size()>0){
                    count = String.valueOf(recordList.get(0).get("cou"));
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDocCreateNum:" + e);
        }

        return count;
    }

    /**
     *
     * @param employee
     * @return
     */
    public String getWorkflowDataAmount(SimpleEmployee employee,int Type) {
        Map<String,Object> data = new HashMap<>(100);
        String count = "0";
        try{
            RequestListConditionApiEntity conditionEntity = new RequestListConditionApiEntity();
//            conditionEntity.setOperatedate(DateTimeConditionEnum.CURRENT_YEAR);
            WeaResult<RequestCountInfoPAEntity>  requestCountInfo = wflRequestListRest.getRequestCountByTabId(employee, Type, conditionEntity);
            int code = requestCountInfo.getCode();
            if(code == 200){
                RequestCountInfoPAEntity requestCountInfoPAEntity = requestCountInfo.getData();
                count = String.valueOf(requestCountInfoPAEntity.getCount());
            }
        } catch (Exception e) {
            log.error("PortalWfDealCountCmd-Exception:" + e);

        }
        return count;
    }


}
