package com.weaver.seconddev.cockpit.cmd;

import com.weaver.common.base.entity.result.WeaResult;
import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.ebuilder.datasource.api.enums.SqlParamType;
import com.weaver.framework.rpc.annotation.RpcReference;
import com.weaver.seconddev.cockpit.util.DatabaseUtils;
import com.weaver.seconddev.cockpit.util.JucailinCockpitUtils;
import com.weaver.seconddev.jcldoor.util.CommonUtils;
import com.weaver.teams.domain.user.SimpleEmployee;
import com.weaver.workflow.common.constant.list.api.DateTimeConditionEnum;
import com.weaver.workflow.common.constant.list.listdimension.RequestListInitDimensionTabEnum;
import com.weaver.workflow.common.entity.list.api.RequestListConditionApiEntity;
import com.weaver.workflow.common.entity.list.api.publicapi.RequestCountInfoPAEntity;
import com.weaver.workflow.common.entity.list.api.publicapi.RequestListInfoPAEntity;
import com.weaver.workflow.list.api.rest.publicapi.WflRequestListRest;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.*;

/**
 * 流程、日程、协助、文档 4个类型数据
 *
 * @author shil
 * @version 1.00版本
 * @Date 2024/5/22
 */
@Component
public class CockpitWorkflowAnalysisCmd {
    //日志
    private final static Logger log = LoggerFactory.getLogger(CockpitWorkflowAnalysisCmd.class);

    @Autowired
    private JucailinCockpitUtils jucailinCockpitUtils;

    @RpcReference(group = "workflow")
    WflRequestListRest wflRequestListRest;

    @Autowired
    private DatabaseUtils databaseUtils;

    public Map<String, Object> getDataInfo(SimpleEmployee employee, Map<String, Object> paramMap,String tenantKey) {

        Map<String, Object> recordMap = new HashMap<String, Object>();
        try {
            List<Map<String,String>> dataList = getWorkflowDate(employee,tenantKey);
            recordMap.put("leftdata",dataList);
            List<Map<String,Object>> rightList = getRightViewData(employee,tenantKey);
            recordMap.put("rightdata",rightList);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("PortalPerformanceInfoCmd :" + e);
        }
        return recordMap;
    }

    /***
     *
     * @return
     */
    public List<Map<String,String>> getWorkflowDate(SimpleEmployee employee,String tenantKey) {

        List<Map<String,String>> wflist = new ArrayList<>(100);
        try {
            Map<String,String> wfMap = new HashMap<String,String>();
            String wf_count1 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.TODO_FLOWALL.getTabid());
            String wf_name1 = "待处理流程数量";
            String wf_pkey1 = "workflow.todo.amount";
            String wf_url1 =  jucailinCockpitUtils.getUfPropData(wf_pkey1,tenantKey);
            wfMap.put("count",wf_count1);
            wfMap.put("name",wf_name1);
            wfMap.put("url",wf_url1);
            wfMap.put("key","dcllcsl");
            wflist.add(wfMap);

            wfMap = new HashMap<String,String>();
            String wf_count2 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.MINE_FLOWALL.getTabid());
            String wf_name2 = "我创建的流程";
            String wf_pkey2 = "workflow.mine.amount";
            String wf_url2 =  jucailinCockpitUtils.getUfPropData(wf_pkey2,tenantKey);
            wfMap.put("count",wf_count2);
            wfMap.put("name",wf_name2);
            wfMap.put("url",wf_url2);
            wfMap.put("key","wcjdlc");
            wflist.add(wfMap);

            wfMap = new HashMap<String,String>();
            String wf_count3 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.DONE_FLOWALL.getTabid());
            String wf_name3 = "我处理的流程";
            String wf_pkey3 = "workflow.done.amount";
            String wf_url3 =  jucailinCockpitUtils.getUfPropData(wf_pkey3,tenantKey);
            wfMap.put("count",wf_count3);
            wfMap.put("name",wf_name3);
            wfMap.put("url",wf_url3);
            wfMap.put("key","wcldlc");
            wflist.add(wfMap);


            wfMap = new HashMap<String,String>();
            String wf_count4 = getWorkflowDataAmount(employee,RequestListInitDimensionTabEnum.SUPERVISE_ALL.getTabid());
            String wf_name4 = "我督办的流程";
            String wf_pkey4 = "workflow.supervise.amount";
            String wf_url4 =  jucailinCockpitUtils.getUfPropData(wf_pkey4,tenantKey);
            wfMap.put("count",wf_count4);
            wfMap.put("name",wf_name4);
            wfMap.put("url",wf_url4);
            wfMap.put("key","wdbdlc");
            wflist.add(wfMap);


            wfMap = new HashMap<String,String>();
            Long employeeid = employee.getEmployeeId();
            String wf_count5 = getWatchWorkflowDataAmount(employeeid,tenantKey);
            String wf_name5 = "我关注的流程";
            String wf_pkey5 = "workflow.attention.amount";
            String wf_url5 =  jucailinCockpitUtils.getUfPropData(wf_pkey5,tenantKey);
            wfMap.put("count",wf_count5);
            wfMap.put("name",wf_name5);
            wfMap.put("url",wf_url5);
            wfMap.put("key","wgzdlc");
            wflist.add(wfMap);

            List<Map<String,String>> daylist = getWorkflowDayOperateCount(employee,tenantKey,wf_count3,wf_count2);
            wflist.addAll(daylist);

            wfMap = getWorkflowOperateAverageTime(employee,tenantKey);
            wflist.add(wfMap);


        } catch (Exception e) {
            e.printStackTrace();
            log.error("DevQuestCountCmd-Exception:" + e);
        }
        return wflist;
    }

    /**
     *
     * @param employee
     * @return
     */
    public String getWorkflowDataAmount(SimpleEmployee employee,int Type) {
        String count = "0";
        try{
            RequestListConditionApiEntity conditionEntity = new RequestListConditionApiEntity();
            WeaResult<RequestCountInfoPAEntity>  requestCountInfo = wflRequestListRest.getRequestCountByTabId(employee, Type, conditionEntity);
            int code = requestCountInfo.getCode();
            if(code == 200){
                RequestCountInfoPAEntity requestCountInfoPAEntity = requestCountInfo.getData();
                count = String.valueOf(requestCountInfoPAEntity.getCount());
            }

        } catch (Exception e) {
            log.error("PortalWfDealCountCmd-Exception:" + e);

        }
        return count;
    }


    public String getWatchWorkflowDataAmount(Long employeeId,String tenantKey) {
        String count = "0";
        String groupId = "weaver-doc-service";
        String sourceType = "LOGIC";

        try {
            String sql =" select count(1) as cou \n" +
                        " from my_watch  \n" +
                        " where watcher = ? \n" +
                        " and module='workflow' \n" +
                        " and delete_type = 0 \n" +
                        " and tenant_key=? " ;

            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.LONG);
            sqlParamEntity.setValue(employeeId+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                count = String.valueOf(recordList.get(0).get("cou"));
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("getDocCreateNum:" + e);
        }
        return count;
    }


    /***
     *
     * @param employee
     * @param tenantKey
     * @return
     */
    public Map<String,String> getWorkflowOperateAverageTime(SimpleEmployee employee,String tenantKey) {
        String count = "0.0";
        try{
            RequestListConditionApiEntity conditionEntity = new RequestListConditionApiEntity();
            conditionEntity.setOperatedate(DateTimeConditionEnum.CURRENT_YEAR);

            int wfcount = 0 ;
            WeaResult<RequestCountInfoPAEntity> requestCountInfo = wflRequestListRest.getRequestCountByTabId(employee, RequestListInitDimensionTabEnum.DONE_FLOWALL.getTabid(), conditionEntity);
            log.error("requestCountInfo--:"+requestCountInfo.toString());
            int code1 = requestCountInfo.getCode();
            log.error("code1--:"+code1);
            if(code1 == 200){
                RequestCountInfoPAEntity requestCountInfoPAEntity = requestCountInfo.getData();
                Long wf_count = requestCountInfoPAEntity.getCount();
                log.error("wfcount--:"+wfcount);
                wfcount = wf_count.intValue();
            }

            if(wfcount > 0){
                WeaResult<List<RequestListInfoPAEntity>> requestListData = wflRequestListRest.getRequestListByTabId(employee, RequestListInitDimensionTabEnum.DONE_FLOWALL.getTabid(), 1, wfcount, conditionEntity);
                int code2 = requestListData.getCode();
                if(code2 == 200){
                    Long minutescount = 0L;
                    List<RequestListInfoPAEntity> requestCountInfoPAEntityList = requestListData.getData();
                    int requestLength = requestCountInfoPAEntityList.size() ;
                    log.error("requestLength:"+requestLength);
                    for(int i=0;i<requestLength;i++){
                        RequestListInfoPAEntity requestListInfoPAEntity = requestCountInfoPAEntityList.get(i);
                        LocalDateTime receiveDateTime = requestListInfoPAEntity.getReceiveTime();
                        LocalDateTime operateDateTime = requestListInfoPAEntity.getOperateTime();
//                        log.error("receiveDateTime:"+receiveDateTime);
//                        log.error("operateDateTime:"+operateDateTime);
                        Long minutesDiff = ChronoUnit.MINUTES.between(receiveDateTime, operateDateTime);
//                        log.error("minutesDiff:"+minutesDiff);
                        minutescount += minutesDiff ;
                    }
                    log.error("minutescount:"+minutescount);
                    if(requestLength > 0 && minutescount > 0){
                        double average = minutescount/(wfcount*1.0) ;
                        count = String.valueOf(average);
                    }
                }
            }
        } catch (Exception e) {
            log.error("PortalWfDealCountCmd-Exception:" + e);
        }

        Map<String,String> wfMap = new HashMap<String,String>();
        String wf_name1 = "平均流程处理时间";
        String wf_pkey1 = "workflow.processtime.amount";
        String wf_url1 =  jucailinCockpitUtils.getUfPropData(wf_pkey1,tenantKey);
        wfMap.put("count",count);
        wfMap.put("name",wf_name1);
        wfMap.put("url",wf_url1);
        wfMap.put("key","pjlcclsj");

        return wfMap;
    }


    public List<Map<String,String>> getWorkflowDayOperateCount(SimpleEmployee employee,String tenantKey,String wf_count3,String wf_count2) {
        String count2 = "0.0";
        String count3 = "0.0";
        String diffdays = "1";

        log.error("wf_count3:"+wf_count3);
        log.error("wf_count2:"+wf_count2);

        String groupId = "weaver-ebuilder-form-service";
        String sourceType = "LOGIC";
        log.error("wf_count3:"+wf_count3);
        try{
            long employeeId = employee.getEmployeeId();


            String dataSql =" select a.hiredate,datediff(curdate(),a.hiredate) as diffdays " +
                            " from eteams.employee a   " +
                            " where a.id = ?   " +
                            " and a.tenant_key = ?  " +
                            " and a.status = 'normal'   " +
                            " and a.type = 'inside'";

            log.error("getuserIfo--dataSql:" + dataSql);
            List<String> strlist = new ArrayList<>(100);
            strlist.add(employeeId + "");
            strlist.add(tenantKey);
            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);
            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, dataSql, sqlparam);
            List<Map<String, Object>> recordList = databaseUtils.getDataSourceList(result);
            if (CollectionUtils.isNotEmpty(recordList)) {
                Map<String, Object> recordMap = recordList.get(0);
                diffdays = CommonUtils.null2String(recordMap.get("diffdays"));
            }

            log.error("diffdays:"+diffdays);

            if(StringUtils.isNotBlank(wf_count3) && StringUtils.isNotBlank(diffdays)){
                Long dealcount = Long.parseLong(wf_count3);
                Long days = Long.parseLong(diffdays);

                BigDecimal bg = new BigDecimal((double) dealcount/(days*1.00));
                count3 = String.valueOf(bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            }

            if(StringUtils.isNotBlank(wf_count2) && StringUtils.isNotBlank(diffdays)){
                Long dealcount = Long.parseLong(wf_count2);
                Long days = Long.parseLong(diffdays);

                BigDecimal bg = new BigDecimal((double) dealcount/(days*1.00));
                count2 = String.valueOf(bg.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue());
            }
        } catch (Exception e) {
            log.error("PortalWfDealCountCmd-Exception:" + e);
        }

        List<Map<String,String>> dayList = new ArrayList<Map<String,String>>();
        Map<String,String> wfMap = new HashMap<String,String>();
        String wf_name2 = "平均每日发起流程";
        String wf_pkey2 = "workflow.daily.create.amount";
        String wf_url2 =  jucailinCockpitUtils.getUfPropData(wf_pkey2,tenantKey);
        wfMap.put("count",count2);
        wfMap.put("name",wf_name2);
        wfMap.put("url",wf_url2);
        wfMap.put("key","pjmrfqlc");
        dayList.add(wfMap);

        wfMap = new HashMap<String,String>();
        String wf_name3 = "平均每日处理流程";
        String wf_pkey3 = "workflow.daily.process.amount";
        String wf_url3 =  jucailinCockpitUtils.getUfPropData(wf_pkey3,tenantKey);
        wfMap.put("count",count3);
        wfMap.put("name",wf_name3);
        wfMap.put("url",wf_url3);
        wfMap.put("key","pjmrcllc");
        dayList.add(wfMap);

        return dayList;
    }

    /***
     *
     * @param employee
     * @param tenantKey
     * @return
     */
    public List<Map<String,Object>> getRightViewData(SimpleEmployee employee,String tenantKey){
        List<Map<String,Object>> recordList = new ArrayList<Map<String,Object>>();
        String groupId = "weaver-workflow-report-serviceworkflowreport";
        String sourceType = "LOGIC";
        Long employeeId = employee.getEmployeeId();
        log.error("employeeId2:"+employeeId);
        try {
            String sql =" select w.workflowid,w.workflowname,w.cou " +
                        " from (\n" +
                        "       select p.workflowid,p.workflowname,p.cou\n" +
                        "       from (\n" +
                        "           select t.workflowid,p.workflowname,count(1) as cou \n" +
                        "           from wfc_requestbase t\n" +
                        "           inner join wfp_base p on p.id = t.workflowid\n" +
                        "           where t.userid = ? \n" +
                        "           and t.delete_type=0\n" +
                        "           and p.delete_type=0\n" +
                        "           and t.tenant_key= ? \n" +
                        "           and p.tenant_key = ?" +
                        "           group by t.workflowid,p.workflowname\n" +
                        "       ) p order by p.cou desc \n" +
                        " ) w limit 5" ;

            log.error("sql2:"+sql);
            List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
            SqlParamEntity sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.LONG);
            sqlParamEntity.setValue(employeeId+"");
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            sqlParamEntity = new SqlParamEntity();
            sqlParamEntity.setParamType(SqlParamType.VARCHAR);
            sqlParamEntity.setValue(tenantKey);
            sqlparam.add(sqlParamEntity);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            recordList = databaseUtils.getDataSourceList(result);

        } catch (Exception e) {
            e.printStackTrace();
            log.error("getRightViewData2:" + e);
        }
        return recordList;
    }







}
