package com.weaver.seconddev.cockpit.util;

import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.ebuilder.datasource.api.enums.SqlParamType;
import com.weaver.ebuilder.form.client.entity.data.*;
import com.weaver.ebuilder.form.client.service.data.RemoteSimpleDataService;
import com.weaver.framework.rpc.annotation.RpcReference;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

@Component
public class JucailinCockpitUtils {

    private final Logger log = LoggerFactory.getLogger(JucailinCockpitUtils.class);


    @Autowired
    private DatabaseUtils databaseUtils;


    @RpcReference(group="ebuilderform")
    private RemoteSimpleDataService remoteSimpleDataService;



    /***
     *
     * @param pkey
     * @param tenant_key
     * @return
     */
    public String getUfPropData(String pkey,String tenant_key) {
        String pvalue = "" ;
        String groupId = "weaver-ebuilder-form-service";
        String sourceType = "LOGIC";
        try{
            if(StringUtils.isNotBlank(pkey))
            {
                String dataSql =" select pvalue \n" +
                        " from uf_nj_prop \n" +
                        " where pkey = ? \n" +
                        " and delete_type = 0 \n" +
                        " and tenant_key='"+tenant_key+"'" ;

                List<SqlParamEntity> sqlparam = new ArrayList<SqlParamEntity>();
                SqlParamEntity sqlParamEntity = new SqlParamEntity();
                sqlParamEntity.setParamType(SqlParamType.VARCHAR);
                sqlParamEntity.setValue(pkey);
                sqlparam.add(sqlParamEntity);

                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, dataSql,sqlparam);
                List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);

                if(CollectionUtils.isNotEmpty(recordList) && recordList.size() >0 ) {
                    Map<String,Object> recordMap = recordList.get(0);
                    if(recordMap.containsKey("pvalue")){
                        pvalue = String.valueOf(recordMap.get("pvalue"));
                    }
                }
            }
        }catch (Exception e){
            e.printStackTrace();
            log.error("getUfPropData-Exception:"+e);
        }
        return pvalue;
    }


    public void saveModeData(String objId,String operator,String tenant_key,List<EBDataReqDto> datas){

        log.error("saveModeData-objid:"+objId);
        log.error("saveModeData-operator:"+operator);
        log.error("saveModeData-tenant_key:"+tenant_key);
        log.error("saveModeData-datas:"+datas.size());
        EBDataChangeReqDto ebDataChangeReqDto = new EBDataChangeReqDto();
        // 构建基础参数; objId 表单id, operator 操作人, tenantKey 租户
        ebDataChangeReqDto.setHeader(new EBDataReqHeader(objId, operator,tenant_key));

        ebDataChangeReqDto.setDatas(datas);
        EBDataChangeResult insertEbDataChangeResult = remoteSimpleDataService.saveFormData(ebDataChangeReqDto);
        log.error("insertEbDataChangeResult:"+insertEbDataChangeResult.getMessage());
        log.error("insertEbDataChangeResult:"+insertEbDataChangeResult.toString());
        log.error("insertEbDataChangeResult:"+insertEbDataChangeResult.getStatus());
    }


    /***
     *
     */
    public void updateModeDataById(String objId,String operator,String tenant_key,List<EBDataReqDto> datas,boolean isNeedAdd){
        /**
         * 直接通过id更新数据， 即update table set a = '11' where id = 1
         */
        EBDataChangeReqDto ebDataChangeReqDto = new EBDataChangeReqDto();
        // 构建基础参数; objId 表单id, operator 操作人, tenantKey 租户
        ebDataChangeReqDto.setHeader(new EBDataReqHeader(objId, operator, tenant_key));
        // 请求操作信息
        EBDataReqOperation ebDataReqOperation = new EBDataReqOperation();
        ebDataReqOperation.setUpdateType(EBDataUpdateType.ids);
        // 数据没有找到的时候是否新增数据
        EBDataReqOperationInfo ebDataReqOperationInfo = new EBDataReqOperationInfo();
        ebDataReqOperationInfo.setNeedAdd(isNeedAdd);
        ebDataReqOperation.setMainData(ebDataReqOperationInfo);
        ebDataChangeReqDto.setOperation(ebDataReqOperation);

        ebDataChangeReqDto.setDatas(datas);
        EBDataChangeResult updateEbDataChangeResult = remoteSimpleDataService.updateFormData(ebDataChangeReqDto);

        log.error("updateEbDataChangeResult:"+updateEbDataChangeResult.getMessage());
        log.error("updateEbDataChangeResult:"+updateEbDataChangeResult.toString());
        log.error("updateEbDataChangeResult:"+updateEbDataChangeResult.getStatus());
    }


    /***
     *
     * @param request
     * @return
     */
    public Map<String, Object> request2Map(HttpServletRequest request) {
        // 参数Map
        Map properties = request.getParameterMap();
        // 返回值Map
        Map<String, Object> returnMap = new HashMap<String, Object>();
        Iterator<Map.Entry> entries = properties.entrySet().iterator();
        Map.Entry entry;
        String name = "";
        Object value = null;
        while (entries.hasNext()) {
            entry = (Map.Entry) entries.next();
            name = (String) entry.getKey();
            Object valueObj = entry.getValue();
            if (null == valueObj) {
                value = null;
            } else if (valueObj instanceof String[]) {
                String[] values = (String[]) valueObj;
                if (values.length == 1) {
                    value = values[0];
                } else {
                    value = values;
                }
            } else {
                value = valueObj.toString();
            }
            returnMap.put(name, value);
        }
        return returnMap;
    }


}
