package com.weaver.seconddev.jcldoor.cmd;

import com.weaver.common.base.entity.result.WeaResult;
import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.ebuilder.datasource.api.enums.SqlParamType;
import com.weaver.framework.rpc.annotation.RpcReference;
import com.weaver.seconddev.jcldoor.util.DatabaseUtils;
import com.weaver.teams.domain.user.SimpleEmployee;
import com.weaver.teams.security.context.UserContext;
import com.weaver.workflow.common.constant.list.api.DateTimeConditionEnum;
import com.weaver.workflow.common.constant.list.listdimension.RequestListInitDimensionTabEnum;
import com.weaver.workflow.common.entity.list.api.RequestListConditionApiEntity;
import com.weaver.workflow.common.entity.list.api.publicapi.RequestCountInfoPAEntity;
import com.weaver.workflow.list.api.rest.publicapi.WflRequestListRest;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAdjusters;
import java.util.*;

/**
 * 人员本年综合信息（沟通协作数+文档贡献度+处理流程数+微博数量）
 *
 * @author wangj
 * @version 1.00版本
 * @Date 2024/3/13
 */
@Component
public class PortalCollaborationInfoCmd {
    //日志
    private final static Logger log = LoggerFactory.getLogger(PortalCollaborationInfoCmd.class);

    @Autowired
    private DatabaseUtils databaseUtils;

    public String tenant_key = "t7akvdnf84" ;

    @RpcReference(group = "workflow")
    WflRequestListRest wflRequestListRest;

    public Map<String, Object> getInfo(HttpServletRequest request, SimpleEmployee simpleEmployee) {

        Map<String,Object> dataMap = new HashMap<String,Object>();
        String startday = request.getParameter("startday");
        String endday = request.getParameter("endday");
        String employeeId = request.getParameter("employeeId");
        int wb = 0;
        int wdgx = 0;
        int gtxz = 0;
        Long lccl = 0L;
        try{
            Map<String,Object> wbMap = getDailyInfo(startday,endday,employeeId);
            wb = (int) wbMap.get("count");

            Map<String,Object> wdMap = getDocInfo(startday,endday,employeeId);
            wdgx = (int) wdMap.get("count");

            Map<String,Object> gtxzMap = getInteractInfo(startday,endday,employeeId);
            gtxz = (int) gtxzMap.get("count");

            Map<String,Object> wfMap = getWfInfo();
            lccl = (Long) wfMap.get("count");



        }catch (Exception e){
            log.error("PortalCollaborationInfoCmd-Exception:"+e.getMessage());
        }
        dataMap.put("lccl",lccl);
        dataMap.put("gtxz",gtxz);
        dataMap.put("wdgx",wdgx);
        dataMap.put("wb",wb);
        return dataMap;
    }



    public Map<String, Object> getDailyInfo(String startday,String endday,String employeeId) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        String groupId = "weaver-blog-service";
        String sourceType = "LOGIC";

        Map<String,Object> dataMap = new HashMap<String,Object>();
        try{


            Calendar calendar = Calendar.getInstance();
            int year = calendar.get(Calendar.YEAR);

            if(StringUtils.isBlank(startday)){
                startday = year+"-01-01";
            }
            if(StringUtils.isBlank(endday)){
                endday = sdf.format(new Date());
            }

            if(StringUtils.isBlank(employeeId)){
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }

            String sql =" select count(1) as count\n" +
                    "    from blog " +
                    "    where creator=?\n" +
                    "    and left(BLOG_TIME,10) >=?\n" +
                    "    and left(BLOG_TIME,10) <=?\n" +
                    "    and type='blog'\n" +
                    "    and TENANT_KEY = ?\n" +
                    "    and MODULE = 'blog'\n" +
                    "    and DELETE_TYPE = 0\n" +
                    "    and CONTENT is not null" ;

            List<String> strlist = new ArrayList<>(100);
            strlist.add(employeeId);
            strlist.add(startday);
            strlist.add(endday);
            strlist.add(tenant_key);

            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                dataMap = recordList.get(0);
            }

        }catch (Exception e){
            dataMap.put("count",0);
        }
        return dataMap;
    }


    /**
     * 本年文档创建贡献数
     * @param
     * @return
     */
    public Map<String, Object> getDocInfo(String startday,String endday,String employeeId) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        String groupId = "weaver-doc-service";
        String sourceType = "LOGIC";


        Map<String,Object> dataMap = new HashMap<String,Object>();
        try{

            Calendar calendar = Calendar.getInstance();
            int year = calendar.get(Calendar.YEAR);

            if(StringUtils.isBlank(startday)){
                startday = year+"-01-01";
            }
            if(StringUtils.isBlank(endday)){
                endday = sdf.format(new Date());
            }

            if(StringUtils.isBlank(employeeId)){
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }

            String sql = " select count(1) as count from document " +
                    " where CREATOR=?\n" +
                    " and left(create_time,10)>=? and left(create_time,10) <= ?" ;

            log.error("PortalDocCreateCountCmd sql:"+sql);
            List<String> strlist = new ArrayList<>(100);
            strlist.add(employeeId);
            strlist.add(startday);
            strlist.add(endday);

            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);

            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                dataMap = recordList.get(0);
            }

        }catch (Exception e){
            dataMap.put("count",0);
        }
        return dataMap;
    }


    /**
     * 本年沟通协作数
     * @param
     * @return
     */
    public Map<String, Object> getInteractInfo(String startday,String endday,String employeeId) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

        String groupId = "weaver-cowork-service";
        String sourceType = "LOGIC";

        Map<String,Object> dataMap = new HashMap<String,Object>();
        try{

            Calendar calendar = Calendar.getInstance();
            int year = calendar.get(Calendar.YEAR);

            if(StringUtils.isBlank(startday)){
                startday = year+"-01-01";
            }
            if(StringUtils.isBlank(endday)){
                endday = sdf.format(new Date());
            }

            if(StringUtils.isBlank(employeeId)){
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }

            String sql = " select count(1) as count from cw_content " +
                    " where  CREATOR=?\n" +
                    " and left(CREATE_TIME,10)>=? and left(CREATE_TIME,10) <= ?" ;

            log.error("PortalInteractCountCmd sql:"+sql);
            List<String> strlist = new ArrayList<>(100);
            strlist.add(employeeId);
            strlist.add(startday);
            strlist.add(endday);

            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);


            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql,sqlparam);
            List<Map<String,Object>> recordList = databaseUtils.getDataSourceList(result);
            if(recordList.size()>0){
                dataMap = recordList.get(0);
            }
        }catch (Exception e){
            e.getMessage();
            dataMap.put("count",0);
        }
        return dataMap;
    }


    /**
     * 本年流程处理数量
     * @param
     * @return
     */
    public Map<String, Object> getWfInfo() {
        log.error("PortalWfDealCountCmd");
        Map<String,Object> data = new HashMap<>(100);

        Long count = 0L ;
        try{
            RequestListConditionApiEntity conditionEntity = new RequestListConditionApiEntity();
            SimpleEmployee employee = UserContext.getCurrentUser();
            conditionEntity.setOperatedate(DateTimeConditionEnum.CURRENT_YEAR);
            WeaResult<RequestCountInfoPAEntity>  requestCountInfo = wflRequestListRest.getRequestCountByTabId(employee, RequestListInitDimensionTabEnum.DONE_FLOWALL.getTabid(), conditionEntity);
            int code = requestCountInfo.getCode();
            if(code == 200){
                RequestCountInfoPAEntity requestCountInfoPAEntity = requestCountInfo.getData();
                count = requestCountInfoPAEntity.getCount();
            }
        } catch (Exception e) {
            log.error("PortalWfDealCountCmd-Exception:" + e);

        }
        data.put("count", count);
        return data;
    }

}
