package com.weaver.seconddev.jcldoor.cmd;

import com.weaver.ebuilder.datasource.api.entity.SqlParamEntity;
import com.weaver.framework.rpc.annotation.RpcReference;
import com.weaver.seconddev.jcldoor.util.DatabaseUtils;
import com.weaver.seconddev.jcldoor.util.JucailinPortalUtils;
import com.weaver.teams.domain.hr.entity.FlowData;
import com.weaver.teams.domain.hr.entity.FlowDataListReq;
import com.weaver.teams.domain.hr.entity.FlowDataListRes;
import com.weaver.teams.domain.user.SimpleEmployee;
import com.weaver.teams.security.context.UserContext;
import com.weaver.workrelate.performance.api.rest.RemotePerformanceService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.Month;
import java.time.YearMonth;
import java.time.temporal.TemporalAdjusters;
import java.util.*;

/**
 * 绩效信息
 *
 * @author wangj
 * @version 1.00版本
 * @Date 2024/3/8
 */
@Component
public class PortalPerformanceInfoCmd {
    //日志
    private final static Logger log = LoggerFactory.getLogger(PortalPerformanceInfoCmd.class);

    @Autowired
    private DatabaseUtils databaseUtils;

    public String tenant_key = "t7akvdnf84";

    @Autowired
    private JucailinPortalUtils jucailinPortalUtils;

    @RpcReference
    public RemotePerformanceService remotePerformanceService;

    public Map<String, Object> getInfo(HttpServletRequest request, SimpleEmployee simpleEmployee) {
        Map<String, Object> dataMap = new HashMap<String, Object>(100);
        try {

            String employeeId = request.getParameter("employeeId");
            LocalDate currentDate = LocalDate.now();
            LocalDate lastMonthDate = currentDate.minusMonths(1);
            int lastMonth = lastMonthDate.getMonthValue();

            if (StringUtils.isBlank(employeeId)) {
                employeeId = String.valueOf(UserContext.getCurrentEmployeeId());
            }

            String groupId = "weaver-ebuilder-form-service";
            String sourceType = "LOGIC";

            Map<String, Object> scoreMap = getLastMonthKpi2(employeeId, groupId, sourceType);

            List<Map<String, String>> yearList = getWholeYearKpi2();

            dataMap.put("scoreMap", scoreMap);
            dataMap.put("yearList", yearList);

        } catch (Exception e) {
            e.printStackTrace();
            log.error("PortalPerformanceInfoCmd :" + e);
            dataMap.put("count", "-1");
        }

        return dataMap;
    }

    public Map<String, Object> getLastMonthKpi2(String employeeId, String groupId, String sourceType) {

        log.error("PortalPerformanceInfoCmd getLastMonthKpi2");

        Map<String, Object> dataMap = new HashMap<String, Object>();

        List<Map<String, Object>> indicatorList = new ArrayList<Map<String, Object>>();

        if (StringUtils.isNotBlank(employeeId)) {
//            String sql =" select left(k.real_period,10) as real_period,left(k.real_period,7) as period_month,k.kpi_scheme,k.score\n" +
//                        " from (select real_period,kpi_scheme,score\n" +
//                        "       from hr_kpi_flow  \n" +
//                        "       where kpi_user=?  \n" +
//                        "       and module = 'kpiFlow' \n" +
//                        "       and date_type = 'month' \n" +
//                        "       and delete_type = '0' " +
//                        "       and tenant_key=?\n" +
//                        "       order by real_period desc\n" +
//                        ") k  limit 1 ";
//
//            log.error("sql20-1:" + sql);
//
//            List<String> strlist = new ArrayList<>(100);
//            strlist.add(employeeId);
//            strlist.add(tenant_key);
//
//
//            List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);
//            Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql, sqlparam);
//
//            List<Map<String, Object>> recordList = databaseUtils.getDataSourceList(result);
//            if (CollectionUtils.isNotEmpty(recordList)) {
//                Map<String, Object> recordMap = recordList.get(0);
//                if (!recordMap.isEmpty()) {
//                    String score = String.valueOf(recordMap.get("score"));
//                    String kpi_scheme = String.valueOf(recordMap.get("kpi_scheme"));
//                    String real_period = String.valueOf(recordMap.get("real_period"));
//                    String period_month = String.valueOf(recordMap.get("period_month"));
//
////                    dataMap.put("score", score);
////                    dataMap.put("yearmonth", period_month);
//                    dataMap.put("kpi_scheme", kpi_scheme);
////                    dataMap.put("real_period", real_period);
//                }
//            }

            try{
                String startDay = "" ;
                String endDay = "" ;
                String score = "0.0";
                String kpi_scheme = "";
                String real_period = "";

                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM");
                String nowDate = sdf.format(new Date())+"-01";
                Map<String,String> scoreMap = getLastScore(nowDate);
                if(!scoreMap.isEmpty()){
                    score = scoreMap.get("score");
                    real_period = scoreMap.get("real_period");
                    startDay = scoreMap.get("startDay");
                    endDay = scoreMap.get("endDay");
                }
                log.error("startDay:"+startDay);
                log.error("endDay:"+endDay);

                String sql =" select scheme_id " +
                        " from hr_kpi_scheme_flow_log_detail " +
                        " where user_id=? \n" +
                        " and date_type='month' \n" +
                        " order by real_period desc " ;

                List<String> strlist = new ArrayList<String>();
                strlist.add(employeeId);
                strlist.add(startDay);
                strlist.add(endDay);
                List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);
                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql, sqlparam);
                List<Map<String, Object>> recordList = databaseUtils.getDataSourceList(result);
                if (CollectionUtils.isNotEmpty(recordList)) {
                    for(int i=0;i<recordList.size();i++){
                        Map<String, Object> recordMap = recordList.get(i);
                        kpi_scheme = String.valueOf(recordMap.get("scheme_id"));
                        if(StringUtils.isNotBlank(kpi_scheme)){
                            break;
                        }
                    }
                }

                log.error("kpi_scheme:"+kpi_scheme);

                if(StringUtils.isNotBlank(kpi_scheme)){
                    dataMap.put("score", score);
                    dataMap.put("yearmonth", real_period);
                    dataMap.put("real_period", real_period);
                    dataMap.put("kpi_scheme", kpi_scheme);
                    dataMap.put("startDay", startDay);
                    dataMap.put("endDay", endDay);
                }else{
                    sql =" select k.kpi_scheme\n" +
                                " from (select real_period,kpi_scheme,score\n" +
                                "       from hr_kpi_flow  \n" +
                                "       where kpi_user=?  \n" +
                                "       and module = 'kpiFlow' \n" +
                                "       and date_type = 'month' \n" +
                                "       and delete_type = '0' " +
                                "       and tenant_key=?\n" +
                                "       order by real_period desc\n" +
                                ") k  limit 1 ";

                    log.error("sql20-1:" + sql);

                    strlist = new ArrayList<>(100);
                    strlist.add(employeeId);
                    strlist.add(tenant_key);

                    sqlparam = databaseUtils.getSqlParamEntity(strlist);
                    result = databaseUtils.executeForQuery(sourceType, groupId, sql, sqlparam);
                    recordList = databaseUtils.getDataSourceList(result);
                    if (CollectionUtils.isNotEmpty(recordList)) {
                        Map<String, Object> recordMap = recordList.get(0);
                        if (!recordMap.isEmpty()) {
                            kpi_scheme = String.valueOf(recordMap.get("kpi_scheme"));
                        }
                    }
                    dataMap.put("score", score);
                    dataMap.put("yearmonth", real_period);
                    dataMap.put("kpi_scheme", kpi_scheme);
                    dataMap.put("real_period", real_period);
                    dataMap.put("startDay", startDay);
                    dataMap.put("endDay", endDay);
                }
            }catch (Exception e){
                e.printStackTrace();
                log.error("getLastScore-e:"+e);
            }
        }

        if (!dataMap.isEmpty()) {
            String kpi_scheme = String.valueOf(dataMap.get("kpi_scheme"));

            log.error("kpi_scheme:" + kpi_scheme);

            if (StringUtils.isNotBlank(kpi_scheme)) {
                String sql = " select indicator_type,indicator_description,weight \n" +
                            " from hr_kpi_scheme_indicator " +
                            " where scheme_id=? " +
                            " and indicator_mode='quantify'\n" +
                            " and status='on' " +
                            " and delete_type = '0'" +
                            " and tenant_key = ?";

                log.error("sql2:" + sql);

                List<String> strlist = new ArrayList<>(100);
                strlist.add(kpi_scheme);
                strlist.add(tenant_key);


                List<SqlParamEntity> sqlparam = databaseUtils.getSqlParamEntity(strlist);
                Map<String, Object> result = databaseUtils.executeForQuery(sourceType, groupId, sql, sqlparam);

                List<Map<String, Object>> recordList = databaseUtils.getDataSourceList(result);

                log.error("recordList:" + recordList.size());

                if (recordList.size() >= 2) {
                    for (int i = 0; i < 2; i++) {
                        indicatorList.add(recordList.get(i));
                    }
                } else if (recordList.size() == 1) {
                    indicatorList.add(recordList.get(0));

                    Map<String, Object> indicatorMap = new HashMap<String, Object>();
                    sql =   " select name,data_row,property\n" +
                            " from hr_kpi_details_link\n" +
                            " where kpi_details in( " +
                            "           select id\n" +
                            "           from hr_kpi_details " +
                            "           where KPI_SCHEME=?\n" +
                            "           and MODULE='kpiSchemeSetting'\n" +
                            "           and TENANT_KEY=?\n" +
                            "           and DELETE_TYPE = 0" +
                            "        )\n" +
                            "    and module = 'kpiSchemeSetting'\n" +
                            "    and tenant_key = ?\n" +
                            "    and delete_type = '0'\n" +
                            "    and property in('weight', 'describe', 'type')\n" +
                            "    and data_row in ( \n" +
                            "        select p.data_row from (\n" +
                            "            select w.data_row from (\n" +
                            "                select data_row \n" +
                            "                from hr_kpi_details_link\n" +
                            "                where kpi_details in ( " +
                            "                       select id\n" +
                            "                       from hr_kpi_details " +
                            "                       where KPI_SCHEME=?\n" +
                            "                       and MODULE='kpiSchemeSetting'\n" +
                            "                       and TENANT_KEY=?\n" +
                            "                       and DELETE_TYPE = 0" +
                            "                 )\n" +
                            "                    and PROPERTY = 'weight'\n" +
                            "                    and MODULE = 'kpiSchemeSetting'\n" +
                            "                    and TENANT_KEY = ?\n" +
                            "                    and DELETE_TYPE = '0'\n" +
                            "                    order by name desc\n" +
                            "            ) w limit 1 \n" +
                            "        ) p\n" +
                            "    )";
                    log.error("sql3:" + sql);

                    List<String> strlist2 = new ArrayList<>(100);
                    strlist2.add(kpi_scheme);
                    strlist2.add(tenant_key);
                    strlist2.add(tenant_key);
                    strlist2.add(kpi_scheme);
                    strlist2.add(tenant_key);
                    strlist2.add(tenant_key);


                    List<SqlParamEntity> sqlparam2 = databaseUtils.getSqlParamEntity(strlist2);
                    Map<String, Object> result2 = databaseUtils.executeForQuery(sourceType, groupId, sql, sqlparam2);


                    List<Map<String, Object>> recordList2 = databaseUtils.getDataSourceList(result2);
                    log.error("recordList2:" + recordList2.size());
                    for (int i = 0; i < recordList2.size(); i++) {
                        Map<String, Object> recordMap = recordList2.get(i);
                        String name = String.valueOf(recordMap.get("name"));
                        String property = String.valueOf(recordMap.get("property"));

                        log.error("name:" + name);
                        log.error("property:" + property);

                        if ("weight".equalsIgnoreCase(property)) {
                            indicatorMap.put("weight", name);
                        }
                        if ("describe".equalsIgnoreCase(property)) {
                            indicatorMap.put("indicator_description", name);
                        }
                        if ("type".equalsIgnoreCase(property)) {
                            indicatorMap.put("indicator_type", name);
                        }
                    }

                    if (!indicatorMap.isEmpty()) {
                        indicatorList.add(indicatorMap);
                    }

                } else {

                    sql =   "  select w.data_row from (\n" +
                            "    select data_row\n" +
                            "    from hr_kpi_details_link\n" +
                            "    where kpi_details in( " +
                            "           select id\n" +
                            "           from hr_kpi_details " +
                            "           where kpi_scheme =?\n" +
                            "           and MODULE='kpiSchemeSetting'\n" +
                            "           and tenant_key =?\n" +
                            "           and delete_type = '0' " +
                            "        )\n" +
                            "        and property = 'weight'\n" +
                            "        and module = 'kpiSchemeSetting'\n" +
                            "        and tenant_key = ?\n" +
                            "        and delete_type = '0'\n" +
                            "    order by name desc\n" +
                            ") w  limit 2";

                    log.error("sql4:" + sql);

                    List<String> strlist3 = new ArrayList<>(100);
                    strlist3.add(kpi_scheme);
                    strlist3.add(tenant_key);
                    strlist3.add(tenant_key);

                    List<SqlParamEntity> sqlparam3 = databaseUtils.getSqlParamEntity(strlist3);
                    Map<String, Object> result3 = databaseUtils.executeForQuery(sourceType, groupId, sql, sqlparam3);

                    List<Map<String, Object>> recordList3 = databaseUtils.getDataSourceList(result3);
                    log.error("recordList3:" + recordList3.size());

                    for (int k = 0; k < recordList3.size(); k++) {
                        Map<String, Object> recordMap3 = recordList3.get(k);

                        String data_row = String.valueOf(recordMap3.get("data_row"));
                        log.error("data_row:" + data_row);
                        Map<String, Object> indicatorMap = new HashMap<String, Object>();

                        sql =   " select name,data_row,property\n" +
                                " from hr_kpi_details_link\n" +
                                " where kpi_details in( " +
                                "           select id\n" +
                                "           from hr_kpi_details " +
                                "           where kpi_scheme=?\n" +
                                "           and module='kpiSchemeSetting'\n" +
                                "           and TENANT_KEY=?\n" +
                                "           and DELETE_TYPE = 0" +
                                "    )\n" +
                                "    and MODULE = 'kpiSchemeSetting'\n" +
                                "    and TENANT_KEY = ?\n" +
                                "    and DELETE_TYPE = '0'\n" +
                                "    and property in('weight', 'describe', 'type')\n" +
                                "    and data_row = ? ";

                        log.error("sql5:" + sql);

                        List<String> strlist2 = new ArrayList<>(100);
                        strlist2.add(kpi_scheme);
                        strlist2.add(tenant_key);
                        strlist2.add(tenant_key);
                        strlist2.add(data_row);


                        List<SqlParamEntity> sqlparam2 = databaseUtils.getSqlParamEntity(strlist2);
                        Map<String, Object> result2 = databaseUtils.executeForQuery(sourceType, groupId, sql, sqlparam2);

                        List<Map<String, Object>> recordList2 = databaseUtils.getDataSourceList(result2);

                        for (int i = 0; i < recordList2.size(); i++) {

                            Map<String, Object> recordMap = recordList2.get(i);

                            String name = String.valueOf(recordMap.get("name"));
                            String property = String.valueOf(recordMap.get("property"));
                            if ("weight".equalsIgnoreCase(property)) {
                                indicatorMap.put("weight", name);
                            }
                            if ("describe".equalsIgnoreCase(property)) {
                                indicatorMap.put("indicator_description", name);
                            }
                            if ("type".equalsIgnoreCase(property)) {
                                indicatorMap.put("indicator_type", name);
                            }
                        }
                        indicatorList.add(indicatorMap);
                    }
                }
            }
            dataMap.put("indicator", indicatorList);
        }
        return dataMap;
    }


    public List<Map<String, String>> getWholeYearKpi2() {

        List<Map<String, String>> dateList = new ArrayList<Map<String, String>>();
        LocalDate localDate = LocalDate.now();
        int year = localDate.getYear();
        int dayOfYear = 1;
        String[] monthzw = new String[]{"一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"};

        for (int i = 1; i <= 12; i++) {
            Map<String, String> dataMap = new HashMap<>();
            Month month = Month.of(i);
            LocalDate startDay = YearMonth.of(year, month).atDay(dayOfYear); // 计算当前月份的首日
            LocalDate endDay = YearMonth.of(year, month).atEndOfMonth(); // 计算当前月份的最后一天
            String yearMonth = "";
            if (monthzw.length == 12) {
                yearMonth = monthzw[i - 1];
            }
            String score = getScore(startDay + "", endDay + "");
            dataMap.put(yearMonth, score);

            dateList.add(dataMap);
        }
        return dateList;
    }


    public String getScore(String startDay, String endDay) {
        String score = "0.0";
        try {
            FlowDataListReq flowDataListReq = new FlowDataListReq();
            flowDataListReq.setCurrent(1);
            flowDataListReq.setPageSize(20);
            flowDataListReq.setStartTime(startDay + "");
            flowDataListReq.setEndTime(endDay + "");
            flowDataListReq.setDateType("month");
            flowDataListReq.setTenantKey("t7akvdnf84");
            List<Long> userList = new ArrayList<Long>();
            Long employeeId = UserContext.getCurrentEmployeeId();
            userList.add(employeeId);
            flowDataListReq.setUserList(userList);
            FlowData flowData = remotePerformanceService.queryFlowData(flowDataListReq);
            log.error("getWholeYearKpi2-flowData:" + flowData);
            List<FlowDataListRes> flowDataList = flowData.getDataList();
            if (flowDataList.size() > 0) {
                for (int index = 0; index < flowDataList.size(); index++) {
                    FlowDataListRes flowDataListRes = flowDataList.get(index);
                    float Score = flowDataListRes.getScore();
                    score = Score + "";
                }
            }
        } catch (Exception e) {
            log.error("getWholeYearKpi2-getScore-Exception:" + e.getMessage());
            score = "0.0";
        }
        return score;
    }


    /**
     *
     * @return
     */
    public Map<String, String> getLastScore(String nowDay) {

        Map<String, String> dataMap = new HashMap<String, String>();
        String DateType = "month";
        String score = "0.0";
        String period_month = "";

        String startDay = "" ;
        String endDay = "" ;

        try {
            LocalDate date = LocalDate.parse(nowDay);
            for (int i = 1; i <= 12; i++) {
                LocalDate localStartDay = date.minusMonths(i);
                LocalDate localEndDay = localStartDay.with(TemporalAdjusters.lastDayOfMonth()); // 获取本月最后一天

                startDay = localStartDay.toString();
                endDay = localEndDay.toString() ;

                log.error("getLastScore-startDay:"+startDay);
                log.error("getLastScore-endDay:"+endDay);

                FlowDataListReq flowDataListReq = new FlowDataListReq();
                flowDataListReq.setCurrent(1);
                flowDataListReq.setPageSize(20);
                flowDataListReq.setStartTime(startDay);
                flowDataListReq.setEndTime(endDay);
                flowDataListReq.setDateType(DateType);
                flowDataListReq.setTenantKey(tenant_key);
                List<Long> userList = new ArrayList<Long>();
                Long employeeId = UserContext.getCurrentEmployeeId();
                userList.add(employeeId);
                flowDataListReq.setUserList(userList);
                FlowData flowData = remotePerformanceService.queryFlowData(flowDataListReq);
                log.error("getWholeYearKpi2-flowData:" + flowData);
                List<FlowDataListRes> flowDataList = flowData.getDataList();
                if (flowDataList.size() > 0) {
                    FlowDataListRes flowDataListRes = flowDataList.get(0);
                    float Score = flowDataListRes.getScore();
                    score = String.valueOf(Score);
                    period_month = startDay.substring(0,7);
                    break;
                }
            }
        } catch (Exception e) {
            log.error("getWholeYearKpi2-getScore-Exception:" + e.getMessage());
            score = "0.0";
        }

        dataMap.put("score", score);
        dataMap.put("yearmonth", period_month);
        dataMap.put("real_period", period_month);
        dataMap.put("startDay", startDay);
        dataMap.put("endDay", endDay);
        return dataMap;
    }


    public static String null2String(String s, String def) {
        return s == null ? (def == null ? "" : def) : s;

    }
}
